@@ -125,23 +125,35 @@ GridLine::DoPlot(anyOutput *o)
 			if(tmp < pts[2].y) pts[2].y = tmp;
 			if(tmp > pts[3].y) pts[3].y = tmp;
 			}
-		SetMinMaxRect(&rDims, pts[0].x, pts[2].y, pts[1].x, pts[3].y);
+		if(pts[0].x != pts[1].x || pts[0].y != pts[1].y){
+			o->oPolyline(pts, 2);
+			SetMinMaxRect(&rDims, pts[0].x, pts[0].y, pts[1].x, pts[1].y);
+			}
+		if(pts[2].x != pts[3].x || pts[2].y != pts[3].y){
+			o->oPolyline(pts+2, 2);
+			SetMinMaxRect(&rDims, pts[2].x, pts[2].y, pts[3].x, pts[3].y);
+			}
 		IncrementMinMaxRect(&rDims, 3);
-		if(pts[0].x != pts[1].x || pts[0].y != pts[1].y) o->oPolyline(pts, 2);
-		if(pts[2].x != pts[3].x || pts[2].y != pts[3].y) o->oPolyline(pts+2, 2);
 		}
 }
 
 void
 GridLine::DoMark(anyOutput *o, bool mark)
 {
-	if(cpts && mark){
-		InvertLine(cpts, ncpts, &LineDef, &rDims, o, mark);
+	if(mark){
+		if(mo) DelBitmapClass(mo);					mo = 0L;
+		memcpy(&mrc, &rDims, sizeof(RECT));
+		IncrementMinMaxRect(&mrc, 6 + o->un2ix(LineDef.width));
+		mo = GetRectBitmap(&mrc, o);
+		if(type & DL_CIRCULAR) {
+			InvertLine(cpts, ncpts, &LineDef, &rDims, o, mark);
+			}
+		else {
+			if(pts[0].x != pts[1].x || pts[0].y != pts[1].y)InvertLine(pts, 2, &LineDef, &rDims, o, mark);
+			if(pts[2].x != pts[3].x || pts[2].y != pts[3].y)InvertLine(pts+2, 2, &LineDef, &rDims, o, mark);
+			}
 		}
-	else if(cpts) {
-		InvertLine(cpts, ncpts, &LineDef, &rDims, o, mark);
-		}
-
+	else RestoreRectBitmap(&mo, &mrc, o);
 }
 
 bool
@@ -175,12 +187,11 @@ GridLine::Command(int cmd, void *tmpl, anyOutput *o)
 		case MOUSE_LBUP:
 			if(IsInRect(&rDims, p1.x = mev->x, p1.y = mev->y)) {
 				if(cpts && (type & DL_CIRCULAR) && IsCloseToPL(p1, cpts, ncpts)) {
-					o->ShowMark(this, MRK_GODRAW);
+					o->ShowMark(CurrGO = this, MRK_GODRAW);
 					return true;
 					}
 				else if(!(type & DL_CIRCULAR)){
-					o->ShowMark(&rDims, MRK_INVERT);
-					CurrGO = this;
+					o->ShowMark(CurrGO = this, MRK_GODRAW);
 					return true;
 					}
 				}
@@ -225,6 +236,7 @@ GridLine3D::DoMark(anyOutput *o, bool mark)
 	POINT3D *gl;
 
 	if(mark){
+		if(mo) DelBitmapClass(mo);					mo = 0L;
 		if(ls){
 			memcpy(&mrc, &rDims, sizeof(RECT));
 			IncrementMinMaxRect(&mrc, 6 + o->un2ix(LineDef.width));
@@ -417,6 +429,7 @@ void
 GridRadial::DoMark(anyOutput *o, bool mark)
 {
 	if(mark) {
+		if(mo) DelBitmapClass(mo);					mo = 0L;
 		memcpy(&mrc, &rDims, sizeof(RECT));
 		IncrementMinMaxRect(&mrc, 6 + o->un2ix(LineDef.width));
 		mo = GetRectBitmap(&mrc, o);
@@ -533,8 +546,9 @@ void
 Tick::DoMark(anyOutput *o, bool mark)
 {
 	if(mark){
+		if(mo) DelBitmapClass(mo);					mo = 0L;
 		memcpy(&mrc, &rDims, sizeof(RECT));
-		IncrementMinMaxRect(&mrc, 6);
+		IncrementMinMaxRect(&mrc, 6 + (parent && parent->Id == GO_AXIS) ? o->un2ix(((Axis*)parent)->axline.width):o->iLine);
 		mo = GetRectBitmap(&mrc, o);
 		InvertLine(pts, 2, defs.GetOutLine(), &rDims, o, mark);
 		}
@@ -566,17 +580,6 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 		if(name) free(name);			name = 0L;
 		if(ls) delete(ls);				ls = 0L;
 		return true;
-	case CMD_HIDE_MARK:
-		if(!tmpl || !o) return false;
-		if(tmpl == (void*)label){
-			label->DoMark(o, false);
-			return true;
-			}
-		else if(tmpl == (void*)Grid){
-			Grid->DoMark(o, false);
-			return true;
-			}
-		return false;
 	case CMD_SET_TICKSTYLE:
 		flags &= ~0x07;
 		flags |= (0x07 & *((DWORD*)tmpl));
@@ -626,6 +629,9 @@ Tick::Command(int cmd, void *tmpl, anyOutput *o)
 		if(Grid) Grid->Command(cmd, tmpl, o);
 		if(label) label->Command(cmd, tmpl, o);
 		return true;
+	case CMD_TEXTTHERE:
+		if(label && label->Command(cmd, tmpl, o)) return true;
+		return false;
 	case CMD_MOUSE_EVENT:
 		if((flags & AXIS_GRIDLINE) && Grid && Grid->Command(cmd, tmpl, o)) return true;
 		if(label && label->Command(cmd, tmpl, o)) return true;
@@ -1188,24 +1194,6 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 		if(axisLabel) axisLabel->Command(cmd, tmpl, o);
 		if(Ticks) for(i = 0; i < NumTicks; i++) if(Ticks[i]) Ticks[i]->Command(cmd, tmpl, o);
 		return true;
-	case CMD_HIDE_MARK:
-		if(!tmpl || !o) return false;
-		if(tmpl == (void*)axisLabel){
-			axisLabel->DoMark(o, false);
-			return true;
-			}
-		if(Ticks) for(i = 0; i < NumTicks; i++) {
-			if(Ticks[i]) {
-				if(tmpl == (void*)Ticks[i]) {
-					Ticks[i]->DoMark(o, false);
-					return true;
-					}	
-				else if(Ticks[i]->Command(cmd, tmpl, o)) return true;
-				}
-			}
-		if(axisLabel && axisLabel->Id == GO_MLABEL) 
-			if(axisLabel->Command(cmd, tmpl, o)) return true;
-		return false;
 	case CMD_SET_DATAOBJ:
 		Id = GO_AXIS;
 		data = (DataObj*)tmpl;
@@ -1262,6 +1250,12 @@ Axis::Command(int cmd, void *tmpl, anyOutput *o)
 		if(Ticks) for(i = 0; i < NumTicks; i++)
 			if(Ticks[i] && Ticks[i]->Command(cmd, tmpl, o)) return true;
 		return false;
+	case CMD_TEXTTHERE:
+		if(axisLabel && axisLabel->Command(cmd, tmpl, o)) return true;
+		if(Ticks) for(i = 0; i < NumTicks; i++) {
+			if(Ticks[i] && Ticks[i]->Command(cmd, tmpl, o)) return true;
+			}
+		return false;
 	case CMD_SET_AXDEF:
 		if(axis = (AxisDEF*)tmpl) {
 			if(axis && axis->owner == (void*)this) {
@@ -1391,6 +1385,113 @@ Axis::SetTick(long idx, double val, DWORD flags, char *txt)
 	if(l) free(l);
 }
 
+void
+Axis::mkTimeAxis()
+{
+	int nstep, mode;
+	double span, val;
+	rlp_datetime start, step;
+	static char *tick_formats[] = {"y", "Y", "W", "x", "Z.V.", "d", "H:M", "d"};
+
+	span = axis->max - axis->min;
+	memset(&step, 0, sizeof(rlp_datetime));		parse_datevalue(&start, axis->Start);
+	start.hours=start.minutes=start.doy = 0;	start.seconds = val = 0.0;
+	if(span > 60.0) start.dow = 1;
+	if(span > 24000.0) {
+		step.year = start.month = start.dom = 1;
+		nstep = 2 + (int)(span / 364.0);		mode = 0;
+		}
+	else if(span > 700.0) {
+		step.month = start.month = start.dom = 1;
+		nstep = 2 + (int)(span / 28.0);			mode = 1;
+		}
+	else if(span > 300.0) {
+		step.month = start.dom = 1;
+		nstep = 2 + (int)(span / 28.0);			mode = 2;
+		}
+	else if(span > 150.0) {
+		step.month = start.dom = 1;
+		nstep = 2 + (int)(span / 28.0);			mode = 3;
+		}
+	else if(span > 60.0) {
+		step.dom = 1;
+		nstep = 6 + (int)(span/7.0);			mode = 4;
+		}
+	else if(span > 8.0) {
+		step.dom = 1;
+		nstep = 2+(int)(span);					mode = 5;
+		}
+	else if(span > 2.0) {
+		step.hours = 6;
+		nstep = 4+(int)(span*4.0);				mode = 6;
+		}
+	else if(span > 0.5) {
+		step.hours = (span > 0.9 ? 2 : 1);
+		nstep = 4+(int)(span*24.0);				mode = 7;
+		}
+	else if(span > 0.05) {
+		step.minutes = (span > 0.3 ? (span > 0.4 ? 30 : 15) : (span <= 0.1 ? 5 : 10));
+		nstep = 100;							mode = 8;
+		}
+	else if(span > 0.005) {
+		step.minutes = 1;
+		nstep = 100;							mode = 8;
+		}
+	else return;
+	if(nstep < 50) nstep = 50;					add_date(&start, 0L);
+	if(!(Ticks = (Tick**)calloc(nstep, sizeof(Tick*))))return;
+	for(NumTicks = 0; NumTicks < nstep && val <= axis->max;	NumTicks++) {
+		val = date2value(&start);
+		switch(mode) {
+		case 0:
+			if((start.year%10) == 0) SetTick(NumTicks, val, axis->flags, date2text(&start, tick_formats[0]));
+			else NumTicks--;					break;
+		case 1:
+			if(start.month == 1){
+				if(span <= 6000.0 || (span <= 12000.0 && (start.year%5) == 0) || (span <= 24000.0 && (start.year%10) == 0))
+					SetTick(NumTicks, val, axis->flags, date2text(&start, 
+					(span > 3000.0 && span < 12000.0)?tick_formats[0] : tick_formats[1]));
+				else NumTicks--;
+				}
+			else if(span <= 1500.0) SetTick(NumTicks, val, axis->flags | AXIS_MINORTICK, 
+				date2text(&start, tick_formats[4]));
+			else NumTicks--;					break;
+		case 2:
+			SetTick(NumTicks, val, axis->flags, date2text(&start, start.month==1 ? tick_formats[0]:tick_formats[2]));
+			break;
+		case 3:
+			SetTick(NumTicks, val, axis->flags, date2text(&start, tick_formats[3]));
+			break;
+		case 4:
+			if(start.dom == 1)SetTick(NumTicks, val, axis->flags, date2text(&start, tick_formats[3]));
+			else if(start.dow == 1) SetTick(NumTicks, val, axis->flags | AXIS_MINORTICK, date2text(&start, tick_formats[4]));
+			else NumTicks--;					break;
+		case 5:
+			if(start.dow == 1) SetTick(NumTicks, val, axis->flags, date2text(&start, tick_formats[4]));
+			else if(span < 30.0)SetTick(NumTicks, val, axis->flags | AXIS_MINORTICK, date2text(&start, tick_formats[7]));
+			else NumTicks--;					break;
+		case 6:
+			if(start.hours == 0) SetTick(NumTicks, val, axis->flags, date2text(&start, tick_formats[4]));
+			else SetTick(NumTicks, val, axis->flags | AXIS_MINORTICK, date2text(&start, tick_formats[6]));
+			break;
+		case 7:
+			if(start.hours == 0) SetTick(NumTicks, val, axis->flags, date2text(&start, tick_formats[5]));
+			else if((start.hours % 6) == 0) SetTick(NumTicks, val, axis->flags, date2text(&start, tick_formats[6]));
+			else SetTick(NumTicks, val, axis->flags | AXIS_MINORTICK, date2text(&start, tick_formats[6]));
+			break;
+		case 8:
+			if(start.minutes == 0){
+				if(span <= 0.3 || (start.hours %2) == 0) SetTick(NumTicks, val, axis->flags, 
+					date2text(&start, tick_formats[6]));
+				else SetTick(NumTicks, val, axis->flags, "");
+				}
+			else SetTick(NumTicks, val, axis->flags | AXIS_MINORTICK, date2text(&start, tick_formats[6]));
+			break;
+			}
+		add_date(&start, &step);
+		}
+}
+
 void
 Axis::CreateTicks()
 {
@@ -1401,6 +1502,7 @@ Axis::CreateTicks()
 
 	if(axis->min == -HUGE_VAL || axis->min == HUGE_VAL) return;
 	if(axis->max == -HUGE_VAL || axis->max == HUGE_VAL) return;
+	if(axis->min >= axis->max) return;
 	Command(CMD_FLUSH, 0L, 0L);
 	if((axis->flags & 0xf000) == AXIS_LOG) {	//log-axis
 		if(axis->Start > defs.min4log) tmp = log10(axis->Start);
@@ -1435,6 +1537,9 @@ Axis::CreateTicks()
 			}
 		}
 	else {										//linear axis
+		if((axis->flags & 0xf000) == AXIS_DATETIME) {
+			mkTimeAxis();		return;
+			}
 		if(atv && (nstep = atv->Count()) && (Ticks = (Tick**)calloc(nstep+1, sizeof(Tick*)))) {
 			for(NumTicks = i = n = 0; NumTicks < nstep && atv->GetItem(i, &tick_label, &fVal);	i++) {
 				SetTick(NumTicks, fVal, axis->flags, tick_label);
