@@ -38,511 +38,13 @@
 
 extern char TmpTxt[];
 extern Default defs;
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// export to *.wmf file (windows meta file)
-// this code is based on information from the following books
-// G. Born, 'Referenzhandbuch Dateiformate', 
-//     Addison-Wesley ISBN 3-89319-815-6
-// T. Hogan, 'Die PC-referenz fr Programmierer',
-//     Microsoft Press: Systema Verlag GmbH ISBN 3-89390-250-3
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-typedef struct {
-	unsigned short mtType, mtHeader, mtVersion, mtSize0, mtSize1, mtNoObj;
-	unsigned mtMaxRec;
-	unsigned short mtnoPar;
-}wmf_header;
-
-class ExportWMF:public anyOutput {
-public:
-	HatchOut *hgo;
-
-	ExportWMF(GraphObj *g, char *FileName, float res, DWORD flags);
-	~ExportWMF();
-	bool SetLine(LineDEF *lDef);
-	bool SetFill(FillDEF *fill);
-	bool SetTextSpec(TextDEF *set);
-	bool StartPage();
-	bool EndPage();
-	bool oCircle(int x1, int y1, int x2, int y2, char* nam = 0L);
-	bool oPolyline(POINT * pts, int cp, char *nam = 0L);
-	bool oRectangle(int x1, int y1, int x2, int y2, char *nam = 0L);
-	bool oSolidLine(POINT *p);
-	bool oTextOut(int x, int y, char *txt, int cb);
-	bool oPolygon(POINT *pts, int cp, char *nam = 0L);
-
-private:
-	int oFile;
-	unsigned file_size, rec_size;
-	wmf_header header;
-	unsigned short currGDIobj, maxGDIobj;
-	unsigned short hPen, hBrush, hFont;
-	GraphObj *go;
-	char *name;
-
-	unsigned short wmfCreateSolidBrush(DWORD color);
-	unsigned short wmfCreateFontIndirect(short, short, short, short, short, unsigned char,
-		unsigned char, unsigned char, unsigned char, unsigned char, unsigned char,
-		unsigned char, unsigned char, char *);
-	unsigned short wmfCreateSolidPen(DWORD color, unsigned short width);
-	void wmfDeleteObject(unsigned short obj);
-	void wmfEllipse(unsigned short, unsigned short, unsigned short, unsigned short);
-	void wmfPolyline(POINT *pts, unsigned short cp);
-	void wmfPolygon(POINT *pts, unsigned short cp);
-	void wmfRectangle(unsigned short, unsigned short, unsigned short, unsigned short);
-	void wmfSelectObject(unsigned short o);
-	void wmfSetBkColor(DWORD col);
-	void wmfSetBkMode(unsigned m);
-	void wmfSetTextAlign(unsigned a);
-	void wmfSetTextColor(DWORD col);
-	void wmfTextOut(unsigned short, unsigned short, char *, unsigned short);
-};
-
-ExportWMF::ExportWMF(GraphObj *g, char *FileName, float res, DWORD flags)
-{
-	currGDIobj = maxGDIobj = 0;
-	hgo =0L;
-	DeskRect.left = DeskRect.top = 0;
-	DeskRect.right = DeskRect.bottom = 0x4fffffff;
-	dFillCol = 0xffffffffL;
-	hPen = 0xffff, hBrush = 0xffff, hFont = 0xffff;
-	hres = vres = res;
-	go = g;
-	if(FileName)name = (char*)memdup(FileName, (int)strlen(FileName)+1, 0);
-	else name = 0L;
-	oFile = 0;
-	rec_size = 28;
-	header.mtType = 1;
-	header.mtHeader = 9;
-	header.mtVersion = 0x300;
-	header.mtSize0 = 1000;
-	header.mtSize1 = 0;
-	header.mtNoObj = 64;
-	header.mtMaxRec = 100;
-	header.mtnoPar = 0;
-}
-
-ExportWMF::~ExportWMF()
-{
-	if(hgo) delete hgo;
-	if(name) free(name);
-}
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-//overloaded functions from the anyOutput class
-bool
-ExportWMF::SetLine(LineDEF *lDef)
-{
-	unsigned short iw;
-
-	if(hPen == 0xffff || lDef->width != LineWidth || lDef->width != LineWidth || 
-		lDef->pattern != dPattern || lDef->color != dLineCol) {
-		LineWidth = lDef->width;
-		iw = (unsigned short)(0.5 + un2fix(lDef->width));
-		dPattern = lDef->pattern;
-		RLP.finc = 256.0/((float)(un2fix(lDef->patlength*8.0)));
-		RLP.fp = 0.0;
-		if(iLine == iw && dLineCol == lDef->color && hPen != 0xffff) return true;
-		iLine = iw;
-		dLineCol = lDef->color;
-		if(hPen != 0xffff) wmfDeleteObject(hPen);
-		iw = iw > 0 ? iw : 1;
-		hPen = wmfCreateSolidPen(dLineCol, iw);
-		wmfSelectObject(hPen);
-		}
-	return true;
-}
-
-bool
-ExportWMF::SetFill(FillDEF *fill)
-{
-	if(!fill) return false;
-	if((fill->type & 0xff) != FILL_NONE) {
-		if(!hgo) hgo = new HatchOut(this);
-		if(hgo) hgo->SetFill(fill);
-		}
-	else {
-		if(hgo) delete hgo;
-		hgo = 0L;
-		}
-	if(dFillCol != fill->color) {
-		if(hBrush != 0xffff) wmfDeleteObject(hBrush);
-		hBrush = wmfCreateSolidBrush(dFillCol = fill->color);
-		wmfSelectObject(hBrush);
-		}
-	dFillCol2 = fill->color2;
-	return true;
-}
-
-bool
-ExportWMF::SetTextSpec(TextDEF *set)
-{
-	unsigned char lfcs, lfpaf;
-	char *face;
-	bool IsModified, RetVal;
-
-	switch(set->Font){
-	case FONT_HELVETICA:
-	default:
-		lfcs = 0;
-		lfpaf = 2 | 2<<4;
-		face = "Arial";
-		break;
-	case FONT_TIMES:
-		lfcs = 0;
-		lfpaf = 2 | 1<<4;
-		face = "Times New Roman";
-		break;
-	case FONT_COURIER:
-		lfcs = 2;
-		lfpaf = 1 | 3<<4;
-		face = "Courier New";
-		break;
-		}
-	if(!set->iSize && set->fSize > 0.001f) set->iSize = un2iy(set->fSize);
-	if(!set->iSize) return false;
-	if(hFont == 0xffff || TxtSet.iSize != set->iSize || TxtSet.Style != set->Style ||
-		TxtSet.RotBL != set->RotBL || TxtSet.RotCHAR != set->RotCHAR ||
-		TxtSet.Font != set->Font || TxtSet.fSize != set->fSize) IsModified = true;
-	else IsModified = false;
-	RetVal = anyOutput::SetTextSpec(set);
-	if (IsModified && RetVal) {
-		hFont = wmfCreateFontIndirect(TxtSet.iSize, 0, 
-			iround(TxtSet.RotBL*10), iround(TxtSet.RotBL*10), 
-			(TxtSet.Style & TXS_BOLD) ? 700 : 400,
-			(TxtSet.Style & TXS_ITALIC) ? 1 : 0,
-			(TxtSet.Style & TXS_UNDERLINE) ? 1 : 0,
-			0, lfcs, 0, 0, 2, lfpaf, face);
-		}
-	if(hFont != 0xffff) wmfSelectObject(hFont);
-	return true;
-}
-
-bool 
-ExportWMF::StartPage()
-{
-	if(!go) return false;
-	if(name) {
-#ifdef USE_WIN_SECURE
-		if(0 !=(_sopen_s(&oFile, name, _O_RDWR | _O_BINARY | _O_CREAT | _O_TRUNC, _SH_DENYNO,
-			_S_IWRITE | _S_IREAD))) {
-#else
-		if(-1 ==(oFile = open(name, O_RDWR | O_BINARY | O_CREAT | O_TRUNC,
-			S_IWRITE | S_IREAD))) {
-#endif
-			ErrorBox("Could not open output file");
-			return false;
-			}
-		}
-	else {
-		oFile = 2;		//stdout
-		}
-	VPorg.fy = -un2fiy(go->GetSize(SIZE_GRECT_TOP));
-	VPorg.fx = -un2fix(go->GetSize(SIZE_GRECT_LEFT));
-	write(oFile, &header, 18);
-	return true;
-}
-
-bool
-ExportWMF::EndPage()
-{
-	unsigned short end_token[3] = {3, 0, 0};
-
-	write(oFile, &end_token, 6);
-	file_size = lseek(oFile, 0L, SEEK_CUR);
-	lseek(oFile, 0L, SEEK_SET);
-	header.mtSize0 = (file_size>>1)&0xffff;
-	header.mtSize1 = file_size>>17;
-	header.mtNoObj = maxGDIobj;
-	header.mtMaxRec = rec_size;
-	write(oFile, &header, 18);
-	oFile = close(oFile);
-	return true;
-}
-
-bool
-ExportWMF::oCircle(int x1, int y1, int x2, int y2, char* nam)
-{
-	wmfEllipse(x1, y1, x2, y2);
-	if(hgo) return hgo->oCircle(x1, y1, x2, y2);
-	return true;
-}
-
-bool
-ExportWMF::oPolyline(POINT * pts, int cp, char * nam)
-{
-	int i;
-
-	if(cp < 1) return false;
-	if (dPattern) for (i = 1; i < cp; i++) PatLine(pts[i-1], pts[i]);
-	else wmfPolyline(pts, cp);
-	return true;
-}
-
-bool
-ExportWMF::oRectangle(int x1, int y1, int x2, int y2, char *nam)
-{
-	wmfRectangle(x1, y1, x2, y2);
-	if(hgo) return hgo->oRectangle(x1, y1, x2, y2, 0L);
-	return true;
-}
-
-bool
-ExportWMF::oSolidLine(POINT *p)
-{
-	wmfPolyline(p, 2);
-	return true;
-}
-
-bool
-ExportWMF::oTextOut(int x, int y, char *txt, int cb)
-{
-	if(!txt || !txt[0]) return false;
-	if(hFont != 0xffff) wmfSelectObject(hFont);
-	wmfSetTextColor(TxtSet.ColTxt);
-	wmfSetBkColor(TxtSet.ColBg);
-	wmfSetTextAlign(((TxtSet.Align & TXA_HRIGHT) ? 2 : (TxtSet.Align &
-		TXA_HCENTER) ? 6 : 0) | ((TxtSet.Align & TXA_VBOTTOM) ?	8 : 0));
-	wmfSetBkMode(TxtSet.Mode ? 1 : 2);
-	wmfTextOut(x, (TxtSet.Align & TXA_VCENTER) ? y - TxtSet.iSize/2 : y, txt, 
-		(unsigned short)((cb > 0) ? cb : strlen(txt)));
-	return true;
-}
-
-bool
-ExportWMF::oPolygon(POINT *pts, int cp, char *nam)
-{
-	wmfPolygon(pts, cp);
-	if(hgo) return hgo->oPolygon(pts, cp);
-	return true;
-}
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-//templates for wmf file records
-typedef struct{
-	unsigned Size;
-	unsigned short Id;
-	unsigned short Value;
-}wmfObjShort;
-
-typedef struct{
-	unsigned Size;
-	unsigned short Id;
-}wmfObjCol;
-
-typedef struct{
-	unsigned Size;
-	unsigned short Id;
-	unsigned short Style;
-	DWORD Col;
-	unsigned short Hatch;
-}wmfLogBrush;
-
-typedef struct{
-	unsigned Size;
-	unsigned short Id;
-	short lfh, lfw, lfesc, lfori, lfwei;
-	unsigned char lfita, lfund, lfsto, lfcse, lfopre, lfclp, lfqua, lfpaqu;
-	char face[32];
-}wmfLogFont;
-
-typedef struct{
-	unsigned Size;
-	unsigned short Id;
-	unsigned short Style;
-	unsigned Width;
-	DWORD col;
-}wmfLogPen;
-
-typedef struct{
-	unsigned Size;
-	unsigned short Id;
-	unsigned short x1, y1, x2, y2;
-}wmfRect;
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-//the following routines resemble the corresponding GDI calls
-unsigned short
-ExportWMF::wmfCreateSolidBrush(DWORD color)
-{
-	wmfLogBrush lb = {7, 0x2fc, 0, color, 0};
-
-	write(oFile, &lb, 14);
-	currGDIobj++;
-	maxGDIobj = currGDIobj > maxGDIobj ? currGDIobj : maxGDIobj;
-	return currGDIobj-1;
-}
-
-unsigned short
-ExportWMF::wmfCreateFontIndirect(short lfHeight, short lfWidth, short lfEscapement,
-	short lfOrientation, short lfWeight, unsigned char lfItalic, unsigned char lfUnderline,
-	unsigned char lfStrikeOut, unsigned char lfCharSet, unsigned char lfOutPrecision, 
-	unsigned char lfClipPrecision, unsigned char lfQuality, unsigned char lfPitchAndFamily,
-	char *FaceName)
-{
-	wmfLogFont lf = {28, 0x2fb, lfHeight, lfWidth, lfEscapement, lfOrientation, lfWeight,
-		lfItalic, lfUnderline, lfStrikeOut, lfCharSet, lfOutPrecision, lfClipPrecision,
-		lfQuality, lfPitchAndFamily, "Arial"};
-
-	if(FaceName && FaceName[0]) rlp_strcpy(lf.face, 32, FaceName);
-	write(oFile, &lf, 56);
-	currGDIobj++;
-	maxGDIobj = currGDIobj > maxGDIobj ? currGDIobj : maxGDIobj;
-	return currGDIobj-1;
-}
-
-unsigned short
-ExportWMF::wmfCreateSolidPen(DWORD color, unsigned short width)
-{
-	wmfLogPen lp = {8, 0x2fa, 0, width, color};
-
-	write(oFile, &lp, 16);
-	currGDIobj++;
-	maxGDIobj = currGDIobj > maxGDIobj ? currGDIobj : maxGDIobj;
-	return currGDIobj-1;
-}
-
-void
-ExportWMF::wmfDeleteObject(unsigned short obj)
-{
-	wmfObjShort wo = {4, 0x1f0, obj};
-
-	if(currGDIobj == obj+1) {
-//		write(oFile, &wo, 8);
-//		currGDIobj--;
-		}
-}
-
-void
-ExportWMF::wmfEllipse(unsigned short ix1, unsigned short iy1, unsigned short ix2, unsigned short iy2)
-{
-	wmfRect rc = {7, 0x418, iy2, ix2, iy1, ix1};
-	
-	write(oFile, &rc, 14);
-}
-
-void
-ExportWMF::wmfPolyline(POINT *pts, unsigned short cp)
-{
-	wmfObjShort pl = {4+cp*2, 0x325, cp};
-	unsigned short v[2];
-	int i;
-
-	write(oFile, &pl, 8);
-	for(i = 0; i < cp; i++) {
-		v[0] = (unsigned short)pts[i].x;
-		v[1] = (unsigned short)pts[i].y;
-		write(oFile, &v, 4);
-		}
-	if(pl.Size > rec_size) rec_size = pl.Size;
-}
-
-void
-ExportWMF::wmfPolygon(POINT *pts, unsigned short cp)
-{
-	wmfObjShort pl = {4+cp*2, 0x324, cp};
-	unsigned short v[2];
-	int i;
-
-	write(oFile, &pl, 8);
-	for(i = 0; i < cp; i++) {
-		v[0] = (unsigned short)pts[i].x;
-		v[1] = (unsigned short)pts[i].y;
-		write(oFile, &v, 4);
-		}
-	if(pl.Size > rec_size) rec_size = pl.Size;
-}
-
-void
-ExportWMF::wmfRectangle(unsigned short ix1, unsigned short iy1, unsigned short ix2, unsigned short iy2)
-{
-	wmfRect rc = {7, 0x41B, iy2, ix2, iy1, ix1};
-	
-	write(oFile, &rc, 14);
-}
-
-void
-ExportWMF::wmfSelectObject(unsigned short o)
-{
-	wmfObjShort so  = {4, 0x12D, o};
-
-	write(oFile, &so, 8);
-}
-
-void
-ExportWMF::wmfSetBkColor(DWORD col)
-{
-	wmfObjCol co = {5, 0x201};
-
-	write(oFile, &co, 6);
-	write(oFile, &col, 4);
-}
-
-void
-ExportWMF::wmfSetBkMode(unsigned m)
-{
-	wmfObjShort mo = {5, 0x102, m & 0xffff};
-	unsigned short p;
-
-	write(oFile, &mo, 8);
-	p = m>>16;
-	write(oFile, &p, 2);
-}
-
-//cmSetMapMode()
-//cmSetPolyFillMode()
-
-void
-ExportWMF::wmfSetTextAlign(unsigned a)
-{
-	wmfObjShort ao = {5, 0x12E, a & 0xffff};
-	unsigned short p;
-
-	write(oFile, &ao, 8);
-	p = a>>16;
-	write(oFile, &p, 2);
-}
-
-void
-ExportWMF::wmfSetTextColor(DWORD col)
-{
-	wmfObjCol tc = {5, 0x209};
-
-	write(oFile, &tc, 6);
-	write(oFile, &col, 4);
-}
-
-//cmSetWindowExt()
-//cmSetWindowOrg()
-
-void
-ExportWMF::wmfTextOut(unsigned short ix1, unsigned short iy1, char *txt, unsigned short cb)
-{
-	wmfObjShort to  = {6, 0x521, cb};
-	unsigned short le, v[2] = {iy1, ix1};
-
-	le = cb &1 ? cb+1 : cb;
-	to.Size += le>>1;
-	write(oFile, &to, 8);
-	write(oFile, txt, le);
-	write(oFile, &v, 4);	
-	if(to.Size > rec_size) rec_size = to.Size;
-}
+static char *str_ind = "                              ";
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Entry point to export graph to Windows Meta File
 void DoExportWmf(GraphObj *g, char *FileName, float res, DWORD flags)
 {
-	ExportWMF *ex;
-	
-
-	InfoBox("The export of Windos metafile (*.wmf) is deprecated.\n\nThis feature will be removed in future\nversions of RLPlot!\n\n");
-	ex = new ExportWMF(g, FileName, res, flags);
-	if(ex->StartPage()) {
-		g->DoPlot(ex);
-		ex->EndPage();
-		}
-	delete(ex);
+	InfoBox("The export of Windos metafile (*.wmf)\n has been dicontinued.\n\n");
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -573,11 +75,11 @@ public:
 	bool oPolygon(POINT *pts, int cp, char * nam = 0L);
 
 private:
-	int iLineWidth, cb_out;
+	int iLineWidth, cb_out, out_pos, out_size, cb_ind;
+	char *out_buff;
 	bool bUseGroupLine, bOutputPending;
 	GraphObj *go;
-	char *name, indent[80], output[120], tHatchStyle[80];
-	FILE *oFile;
+	char *name, output[120], tHatchStyle[80];
 	DWORD flags;
 
 	bool com_TextOut(int x, int y, char *txt, int cb);
@@ -596,9 +98,8 @@ ExportSVG::ExportSVG(GraphObj *g, char *FileName, DWORD flg)
 	go = g;
 	if(FileName)name = (char*)memdup(FileName, (int)strlen(FileName)+1, 0);
 	else name = 0L;
-	oFile = 0L;
-	flags = flg;
-	rlp_strcpy(indent, 80, "   ");
+	out_buff = 0L;		out_pos = out_size = 0;
+	flags = flg;		cb_ind = 3;
 	rlp_strcpy(tHatchStyle, 80, "style=\"stroke:black; stroke-width:1\"");
 	bUseGroupLine = false;
 }
@@ -671,7 +172,34 @@ ExportSVG::StartPage()
 	w = un2ix(go->GetSize(SIZE_GRECT_RIGHT) - go->GetSize(SIZE_GRECT_LEFT))/10;
 	h = un2iy(go->GetSize(SIZE_GRECT_BOTTOM) - go->GetSize(SIZE_GRECT_TOP))/10;
 	w++; h++;
-	if(name) {
+	if(flags & 0x01) add_to_buff(&out_buff, &out_pos, &out_size, "Content-Type: image/svg+xml\n\n", 0);
+	VPorg.fy = -un2fiy(go->GetSize(SIZE_GRECT_TOP));
+	VPorg.fx = -un2fix(go->GetSize(SIZE_GRECT_LEFT));
+	add_to_buff(&out_buff, &out_pos, &out_size, "<?xml version=\"1.0\"?>\n"
+		"<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20001102//EN\"\n"
+		"   \"http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd\">\n<svg ", 0);
+	if(defs.svgAttr) add_to_buff(&out_buff, &out_pos, &out_size, defs.svgAttr, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, " width=\"", 0);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, w, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" height=\"", 0);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, h, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" style=\"stroke-linecap:round\">\n", 0);
+	if(defs.svgScript) {
+		add_to_buff(&out_buff, &out_pos, &out_size, "<defs>\n<script type=\"text/ecmascript\"><![CDATA[\n\n", 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, defs.svgScript, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "\n\n]]></script>\n</defs>\n\n", 0);
+		}
+	add_to_buff(&out_buff, &out_pos, &out_size, "<g transform=\"scale(0.1)\" style=\"font-family:Helvetica\">\n", 0);
+	return true;
+}
+
+bool
+ExportSVG::EndPage()
+{
+	FILE *oFile;
+
+	add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n</svg>\n", 0);
+	if(name && out_buff && out_pos > 20) {
 #ifdef USE_WIN_SECURE
 		fopen_s(&oFile, name, "w");
 #else
@@ -681,30 +209,9 @@ ExportSVG::StartPage()
 			ErrorBox("Could not open\noutput file!");
 			return false;
 			}
+		fprintf(oFile, "%s", out_buff);
+		free(out_buff);		fclose (oFile);
 		}
-	else oFile = stdout;
-	if(flags & 0x01) fprintf(oFile, "Content-Type: image/svg+xml\n\n");
-	VPorg.fy = -un2fiy(go->GetSize(SIZE_GRECT_TOP));
-	VPorg.fx = -un2fix(go->GetSize(SIZE_GRECT_LEFT));
-	fprintf(oFile, "<?xml version=\"1.0\"?>\n"
-		"<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20001102//EN\"\n"
-		"   \"http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001102.dtd\">\n");
-	fprintf(oFile, "<svg %s width=\"%d\" height=\"%d\" style=\"stroke-linecap:round\">\n", 
-		defs.svgAttr ? defs.svgAttr : "", w, h);
-	if(defs.svgScript) {
-		fprintf(oFile, "<defs>\n<script type=\"text/ecmascript\"><![CDATA[\n");
-		fprintf(oFile, "\n%s\n", defs.svgScript);
-		fprintf(oFile, "\n]]></script>\n</defs>\n\n");
-		}
-	fprintf(oFile, "<g transform=\"scale(0.1)\" style=\"font-family:Helvetica\">\n");
-	return true;
-}
-
-bool
-ExportSVG::EndPage()
-{
-	fprintf(oFile, "</g>\n</svg>\n");
-	fclose (oFile);
 	return true;
 }
 
@@ -715,27 +222,53 @@ ExportSVG::oCircle(int x1, int y1, int x2, int y2, char* nam)
 	if(y1 > y2) Swap(y1, y2);
 
 	if(hgo){
-		fprintf(oFile, "%s<g>  <!-- %s with pattern -->\n", indent, 
-			(x2-x1) == (y2-y1) ? "circle" : "ellipse");
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "<g>  <!-- ", 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, (x2-x1) == (y2-y1) ? (char*)"circle" : (char*)"ellipse", 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, " with pattern -->\n", 0);
 		Indent(true);
 		}
-	fprintf(oFile, "%s<%s%s%s%s cx=\"%d\" cy=\"%d\" ", indent,
-		(x2-x1) == (y2-y1) ? "circle" : "ellipse", nam? " name=\"" : "", 
-		nam ? nam : "", nam ? "\"" : "", (x1+x2)/2, (y1+y2)/2);
-	if((x2-x1) == (y2-y1)) fprintf(oFile, "r=\"%d\"", (x2-x1)/2);
-	else fprintf(oFile, "rx=\"%d\" ry=\"%d\"", (x2-x1)/2, (y2-y1)/2);
-	fprintf(oFile, " style=\"fill:%s; stroke:%s; stroke-width:%d\"/>\n",
-		ColName(dFillCol), ColName(dLineCol), iLineWidth);
+	add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+	add_to_buff(&out_buff, &out_pos, &out_size, (x2-x1) == (y2-y1) ? (char*)"<circle" : (char*)"<ellipse", 0);
+	if(nam && nam[0]) {
+		add_to_buff(&out_buff, &out_pos, &out_size, " name=\"", 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, nam, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "\"", 0);
+		}
+	add_to_buff(&out_buff, &out_pos, &out_size, " cx=\"", 0);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, (x1+x2)>>1, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" cy=\"", 0);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, (y1+y2)>>1, false, 0);
+	if((x2-x1)==(y2-y1)) {
+		add_to_buff(&out_buff, &out_pos, &out_size, "\" r=\"", 0);
+		add_int_to_buff(&out_buff, &out_pos, &out_size, (x2-x1)>>1, false, 0);
+		}
+	else {
+		add_to_buff(&out_buff, &out_pos, &out_size, "\" rx=\"", 0);
+		add_int_to_buff(&out_buff, &out_pos, &out_size, (x2-x1)>>1, false, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "\" ry=\"", 0);
+		add_int_to_buff(&out_buff, &out_pos, &out_size, (y2-y1)>>1, false, 0);
+		}
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" style=\"fill:", 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, ColName(dFillCol), 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "; stroke:", 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, ColName(dLineCol), 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "; stroke-width:", 0);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, iLineWidth, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\"/>\n", 0);
 	if(hgo) {
-		fprintf(oFile, "%s<g %s>  <!-- hatch -->\n", indent, tHatchStyle);
-		Indent(true);
-		bUseGroupLine = true;
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "<g ", 3);
+		add_to_buff(&out_buff, &out_pos, &out_size, tHatchStyle, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, ">  <!-- hatch -->\n", 0);
+		Indent(true);		bUseGroupLine = true;
 		hgo->oCircle(x1, y1, x2, y2);
+		Indent(false);		bUseGroupLine = false;
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n", 0);
 		Indent(false);
-		bUseGroupLine = false;
-		fprintf(oFile, "%s</g>\n", indent);
-		Indent(false);
-		fprintf(oFile, "%s</g>\n", indent);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n", 0);
 		}
 	return true;
 }
@@ -744,32 +277,40 @@ bool
 ExportSVG::oPolyline(POINT *pts, int cp, char *nam)
 {
 	int i, cb;
-	char tmptxt[40];
+	char tmptxt[120];
 
 	if(cp < 2) return false;
 	if (dPattern){
-		fprintf(oFile, "%s<g style=\"stroke:%s; stroke-width:%d; stroke-linecap:round\">"
-			"<!-- pattern line -->\n", indent, ColName(dLineCol), iLineWidth);
-		Indent(true);
-		bUseGroupLine = true;
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "<g style=\"stroke:", 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, ColName(dLineCol), 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "; stroke-width:", 0);
+		add_int_to_buff(&out_buff, &out_pos, &out_size, iLineWidth, false, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "; stroke-linecap:round\"><!-- pattern line -->\n", 0);
+		Indent(true);			bUseGroupLine = true;
 		for (i = 1; i < cp; i++) PatLine(pts[i-1], pts[i]);
 		Indent(false);
-		fprintf(oFile, "%s</g>\n", indent);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n", 0);
 		bUseGroupLine = false;
 		}
 	else {
 		if(cp == 2) return oSolidLine(pts);
 		bOutputPending = false;
-		cb_out = sprintf(output, "<polyline points=\""); 
+		cb_out = rlp_strcpy(output, 20,"<polyline points=\""); 
 		for(i = 0; i < cp; i++) {
+#ifdef USE_WIN_SECURE
+			cb = sprintf_s(tmptxt, 120, "%d %d ", pts[i].x, pts[i].y);
+#else
 			cb = sprintf(tmptxt, "%d %d ", pts[i].x, pts[i].y);
+#endif
 			AddToOutput(tmptxt, cb);
 			}
 		if(cb_out) output[cb_out-1] = '"';
 		if(!bUseGroupLine) {
 			cb = rlp_strcpy(tmptxt, 120, " style = \"fill:none; ");
 			AddToOutput(tmptxt, cb);
-			cb = rlp_strcpy(tmptxt, 120, "; stroke:");
+			cb = rlp_strcpy(tmptxt, 120, "stroke:");	//bug fixed by vefremov
 			cb += rlp_strcpy(tmptxt+cb, 120-cb, ColName(dLineCol));
 			cb += rlp_strcpy(tmptxt+cb, 120-cb, "; ");
 			AddToOutput(tmptxt, cb);
@@ -781,7 +322,9 @@ ExportSVG::oPolyline(POINT *pts, int cp, char *nam)
 			AddToOutput(tmptxt, cb);
 			}
 		else AddToOutput("/>", 2);
-		fprintf(oFile, "%s%s\n", indent, output);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, output, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "\n", 1);
 		if(bOutputPending)Indent(false);
 		}
 	return true;
@@ -793,23 +336,45 @@ ExportSVG::oRectangle(int x1, int y1, int x2, int y2, char *nam)
 	if(x1 > x2) Swap(x1, x2);
 	if(y1 > y2) Swap(y1, y2);
 	if(hgo){
-		fprintf(oFile, "%s<g>  <!-- rectangle with pattern -->\n", indent);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "<g>  <!-- rectangle with pattern -->\n", 0);
 		Indent(true);
 		}
-	fprintf(oFile, "%s<rect%s%s%s x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" "
-		"style=\"fill:%s; stroke:%s; stroke-width:%d\"/>\n",
-		indent, nam? " name=\"" : "", nam ? nam : "", nam ? "\"" : "",
-		x1, y1, x2-x1-1, y2-y1-1, ColName(dFillCol), ColName(dLineCol), iLineWidth);
+	add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+	add_to_buff(&out_buff, &out_pos, &out_size, "<rect", 5);
+	if(nam && nam[0]) {
+		add_to_buff(&out_buff, &out_pos, &out_size, " name=\"", 7);
+		add_to_buff(&out_buff, &out_pos, &out_size, nam, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "\"", 1);
+		}
+	add_to_buff(&out_buff, &out_pos, &out_size, " x=\"", 4);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, x1, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" y=\"", 5);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, y1, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" width=\"", 9);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, x2-x1-1, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" height=\"", 10);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, y2-y1-1, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" style=\"fill:", 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, ColName(dFillCol), 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "; stroke:", 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, ColName(dLineCol), 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "; stroke-width:", 0);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, iLineWidth, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\"/>\n", 0);
 	if(hgo) {
-		fprintf(oFile, "%s<g %s>  <!-- hatch -->\n", indent, tHatchStyle);
-		Indent(true);
-		bUseGroupLine = true;
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "<g ", 3);
+		add_to_buff(&out_buff, &out_pos, &out_size, tHatchStyle, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, ">  <!-- hatch -->\n", 0);
+		Indent(true);				bUseGroupLine = true;
 		hgo->oRectangle(x1, y1, x2, y2, 0L);
+		Indent(false);				bUseGroupLine = false;
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n", 5);
 		Indent(false);
-		bUseGroupLine = false;
-		fprintf(oFile, "%s</g>\n", indent);
-		Indent(false);
-		fprintf(oFile, "%s</g>\n", indent);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n", 5);
 		}
 	return true;
 }
@@ -817,11 +382,23 @@ ExportSVG::oRectangle(int x1, int y1, int x2, int y2, char *nam)
 bool
 ExportSVG::oSolidLine(POINT *p)
 {
-	if(bUseGroupLine) fprintf(oFile, "%s<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\"/>\n",
-		indent, p[0].x, p[0].y, p[1].x, p[1].y);
-	else fprintf(oFile, "%s<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" "
-		"style=\"stroke:%s; stroke-width:%d\"/>\n",
-		indent, p[0].x, p[0].y, p[1].x, p[1].y, ColName(dLineCol), iLineWidth);
+	if(!p) return false;
+	add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+	add_to_buff(&out_buff, &out_pos, &out_size, "<line x1=\"", 0);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, p[0].x, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" y1=\"", 6);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, p[0].y, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" x2=\"", 6);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, p[1].x, false, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\" y2=\"", 6);
+	add_int_to_buff(&out_buff, &out_pos, &out_size, p[1].y, false, 0);
+	if(!bUseGroupLine) {
+		add_to_buff(&out_buff, &out_pos, &out_size, "\" style=\"stroke:", 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, ColName(dLineCol), 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "; stroke-width:", 0);
+		add_int_to_buff(&out_buff, &out_pos, &out_size, iLineWidth, false, 0);
+		}
+	add_to_buff(&out_buff, &out_pos, &out_size, "\"/>\n", 4);
 	return true;
 }
 
@@ -876,24 +453,26 @@ ExportSVG::com_TextOut(int x, int y, char *txt, int cb)
 #endif
 	AddToOutput(tmptxt, c);
 #ifdef USE_WIN_SECURE
-	c = sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", h, 
+	c = sprintf_s(tmptxt, 120, "font-size:%d; text-anchor:%s \">", h, 
 		(TxtSet.Align & TXA_HRIGHT) ? "end" : (TxtSet.Align & TXA_HCENTER) ? "middle":"start");
 #else
 	c = sprintf(tmptxt, "font-size:%d; text-anchor:%s \">", h, 
 		(TxtSet.Align & TXA_HRIGHT) ? "end" : (TxtSet.Align & TXA_HCENTER) ? "middle":"start");
 #endif
 	AddToOutput(tmptxt, c);
-	if((strlen(indent)+strlen(txt)+cb_out) <110) cb_out += rlp_strcpy(output+cb_out, 120-cb_out, txt);
+	if((cb_ind+strlen(txt)+cb_out) <110) cb_out += rlp_strcpy(output+cb_out, 120-cb_out, txt);
 	else {
-		fprintf(oFile, "%s%s\n", indent, output);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, output, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "\n", 1);
 		cb_out=rlp_strcpy(output, 120, txt);
 		}
-	if((strlen(indent) + cb_out) <104) 
-		fprintf(oFile, "%s%s</text>\n", indent, output);
-	else {
-		fprintf(oFile, "%s%s\n", indent, output);
-		fprintf(oFile, "</text>\n");
+	add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+	add_to_buff(&out_buff, &out_pos, &out_size, output, 0);
+	if((cb_ind + cb_out) <104) {
+		add_to_buff(&out_buff, &out_pos, &out_size, "</text>\n", 0);
 		}
+	else add_to_buff(&out_buff, &out_pos, &out_size, "\n</text>\n", 0);
 	return true;
 }
 
@@ -961,7 +540,8 @@ ExportSVG::oPolygon(POINT *pts, int cp, char *nam)
 
 	if(cp <3) return false;
 	if(hgo){
-		fprintf(oFile, "%s<g>  <!-- polygon with pattern -->\n", indent);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "<g>  <!-- polygon with pattern -->\n", 0);
 		Indent(true);
 		}
 	bOutputPending = false;
@@ -982,32 +562,37 @@ ExportSVG::oPolygon(POINT *pts, int cp, char *nam)
 		}
 	if(cb_out) output[cb_out-1] = '"';
 #ifdef USE_WIN_SECURE
-	cb = sprintf(tmptxt, "style=\"fill:%s; ", ColName(dFillCol));
+	cb = sprintf_s(tmptxt, 40, " style=\"fill:%s; ", ColName(dFillCol));
 	AddToOutput(tmptxt, cb);
-	cb = sprintf(tmptxt, "stroke:%s; ", ColName(dLineCol));
+	cb = sprintf_s(tmptxt, 40, "stroke:%s; ", ColName(dLineCol));
 	AddToOutput(tmptxt, cb);
-	cb = sprintf(tmptxt, "stroke-width:%d\"/>",iLineWidth);
+	cb = sprintf_s(tmptxt, 40, "stroke-width:%d\"/>",iLineWidth);
 	AddToOutput(tmptxt, cb);
 #else
-	cb = sprintf(tmptxt, "style=\"fill:%s; ", ColName(dFillCol));
+	cb = sprintf(tmptxt, " style=\"fill:%s; ", ColName(dFillCol));
 	AddToOutput(tmptxt, cb);
 	cb = sprintf(tmptxt, "stroke:%s; ", ColName(dLineCol));
 	AddToOutput(tmptxt, cb);
 	cb = sprintf(tmptxt, "stroke-width:%d\"/>",iLineWidth);
 	AddToOutput(tmptxt, cb);
 #endif
-	if(output)fprintf(oFile, "%s%s\n", indent, output);
+	add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+	add_to_buff(&out_buff, &out_pos, &out_size, output, 0);
+	add_to_buff(&out_buff, &out_pos, &out_size, "\n", 1);
 	if(bOutputPending)Indent(false);
 	if(hgo) {
-		fprintf(oFile, "%s<g %s>  <!-- hatch -->\n", indent, tHatchStyle);
-		Indent(true);
-		bUseGroupLine = true;
-		hgo->oPolygon(pts, cp);
-		Indent(false);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "<g ", 3);
+		add_to_buff(&out_buff, &out_pos, &out_size, tHatchStyle, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, ">  <!-- hatch -->\n", 0);
+		Indent(true);				bUseGroupLine = true;
+		hgo->oPolygon(pts, cp);		Indent(false);
 		bUseGroupLine = false;
-		fprintf(oFile, "%s</g>\n", indent);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n", 5);
 		Indent(false);
-		fprintf(oFile, "%s</g>\n", indent);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, "</g>\n", 5);
 		}
 	return true;
 }
@@ -1015,11 +600,12 @@ ExportSVG::oPolygon(POINT *pts, int cp, char *nam)
 void
 ExportSVG::Indent(bool ind)
 {
-	int i = (int)strlen(indent);
-
-	if(i > 20 && ind) return;
-	if(ind) strcat(indent, "   ");
-	else if(i>2) indent[i-3] = 0;
+	if(ind) {
+		if(cb_ind < 20) cb_ind += 3;
+		}
+	else {
+		if(cb_ind > 5) cb_ind -= 3;
+		}
 }
 
 void
@@ -1027,11 +613,13 @@ ExportSVG::AddToOutput(char *txt, int len)
 {
 	if(!txt || !txt[0]) return;
 	if(!len) len = (int)strlen(txt);
-	if((len + cb_out + strlen(indent)) < 110){
+	if((len + cb_out + cb_ind) < 110){
 		cb_out += rlp_strcpy(output+cb_out, 120, txt);
 		}
 	else {
-		fprintf(oFile, "%s%s\n", indent, output);
+		add_to_buff(&out_buff, &out_pos, &out_size, str_ind, cb_ind);
+		add_to_buff(&out_buff, &out_pos, &out_size, output, 0);
+		add_to_buff(&out_buff, &out_pos, &out_size, "\n", 1);
 		if(!bOutputPending) Indent(true);
 		bOutputPending = true;
 		cb_out = rlp_strcpy(output, 120, txt);
@@ -1043,6 +631,7 @@ ExportSVG::ColName(DWORD col)
 {
 	static char txt1[20], txt2[20];
 	static int sw;
+	char *ret;
 
 	switch(col) {
 	case 0x00000000:			return "black";
@@ -1055,14 +644,12 @@ ExportSVG::ColName(DWORD col)
 	case 0x00ffffff:			return "white";
 		}
 	sw++;
-	if(sw & 0x01) {
-		sprintf(txt1, "rgb(%d,%d,%d)", col & 0xff, (col>>8) & 0xff, (col>>16) &0xff);
-		return txt1;
-		}
-	else {
-		sprintf(txt2, "rgb(%d,%d,%d)", col & 0xff, (col>>8) & 0xff, (col>>16) &0xff);
-		return txt2;
-		}
+#ifdef USE_WIN_SECURE
+	sprintf_s(ret = (sw & 0x01 ? txt1 : txt2), 20, "rgb(%d,%d,%d)", col & 0xff, (col>>8) & 0xff, (col>>16) &0xff);
+#else
+	sprintf(ret = (sw & 0x01 ? txt1 : txt2), "rgb(%d,%d,%d)", col & 0xff, (col>>8) & 0xff, (col>>16) &0xff);
+#endif
+	return ret;
 }
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -1127,7 +714,8 @@ ExportEPS::ExportEPS(GraphObj *g, char *FileName, DWORD flags)
 	DeskRect.right = DeskRect.bottom = 0x4fffffff;
 	dFillCol = 0xffffffffL;					hres = vres = 720.0f;
 	go = g;			bFontChange = false;	oFile = 0L;
-	if(FileName)name = strdup(FileName);	else name = 0L;
+	if(FileName)name = (char*)memdup(FileName, (int)strlen(FileName)+1,0);
+	else name = 0L;
 }
 
 ExportEPS::~ExportEPS()
@@ -1200,7 +788,11 @@ ExportEPS::StartPage()
 	BoundBox.bottom = un2iy(go->GetSize(SIZE_GRECT_BOTTOM) - go->GetSize(SIZE_GRECT_TOP))/10;
 	BoundBox.right++;	BoundBox.bottom++;
 	if(name) {
+#ifdef USE_WIN_SECURE
+		fopen_s(&oFile, name, "w");
+#else
 		oFile = fopen(name, "w");
+#endif
 		if(!oFile) {
 			ErrorBox("Could not open\noutput file!");
 			return false;
@@ -1214,7 +806,12 @@ ExportEPS::StartPage()
 		BoundBox.right, BoundBox.bottom);
 	fprintf(oFile, "%%%%Title: %s\n", name);
 	fprintf(oFile,"%%%%Creator: RLPlot version "SZ_VERSION"\n");
+#ifdef USE_WIN_SECURE
+	ctime_s(TmpTxt, 50, &ti);	TmpTxt[24] = 0;
+	fprintf(oFile,"%%%%CreationDate: %s", TmpTxt);
+#else
 	fprintf(oFile,"%%%%CreationDate: %s", ctime(&ti));
+#endif
 	fprintf(oFile, "%%%%Pages: 1\n%%%%DocumentFonts: (atend)\n");
 	fprintf(oFile, "%%%%EndComments\n"
 		"%%%%BeginProlog\n"
