@@ -729,16 +729,31 @@ void OD_PlotTempl(int cmd, void *par, RECT *rec, anyOutput *o,
 			o->oRectangle(rec->left+29, rec->bottom-35, rec->left+33, rec->bottom-3);
 			o->oRectangle(rec->left+36, rec->top+30, rec->left+40, rec->bottom-3);
 			break;
-		case 504:
+		case 504:	case 507:
 			o->SetFill(&FillR);
 			o->oRectangle(rec->left+9, rec->top+30, rec->left+13, rec->bottom-3);
-			o->oRectangle(rec->right-20, rec->top+10, rec->right-16, rec->bottom-3);
+			o->oRectangle(rec->right-20, id == 507 ? rec->top + 25 : rec->top+10, rec->right-16, rec->bottom-3);
 			o->SetFill(&FillG);
 			o->oRectangle(rec->left+13, rec->top+25, rec->left+17, rec->bottom-3);
 			o->oRectangle(rec->right-16, rec->top+15, rec->right-12, rec->bottom-3);
 			o->SetFill(&FillB);
 			o->oRectangle(rec->left+17, rec->top+35, rec->left+21, rec->bottom-3);
 			o->oRectangle(rec->right-12, rec->top+20, rec->right-8, rec->bottom-3);
+			if(id == 507) {
+				o->SetLine(&Line);
+				pts[0].x = pts[1].x = rec->left+11;			pts[0].y = rec->top+20;
+				pts[1].y = rec->top+40;						o->oSolidLine(pts);
+				pts[0].x = pts[1].x = rec->left+15;			pts[0].y = rec->top+15;
+				pts[1].y = rec->top+35;						o->oSolidLine(pts);
+				pts[0].x = pts[1].x = rec->left+19;			pts[0].y = rec->top+30;
+				pts[1].y = rec->top+40;						o->oSolidLine(pts);
+				pts[0].x = pts[1].x = rec->right-18;		pts[0].y = rec->top+15;
+				pts[1].y = rec->top+35;						o->oSolidLine(pts);
+				pts[0].x = pts[1].x = rec->right-14;		pts[0].y = rec->top+10;
+				pts[1].y = rec->top+20;						o->oSolidLine(pts);
+				pts[0].x = pts[1].x = rec->right-10;		pts[0].y = rec->top+10;
+				pts[1].y = rec->top+30;						o->oSolidLine(pts);
+				}
 			break;
 		case 505:
 			o->SetFill(&FillY);
