@@ -408,17 +408,15 @@ anyOutput::ShowMark(void *src, int Mode)
 	GraphObj *go;
 
 	if(MrkMode != MRK_NONE) HideMark();
-	MrkMode = Mode;			MrkRect = src;
+	MrkMode = Mode;			MrkRect = src;		HideCopyMark();
 	switch (Mode) {
 		case MRK_INVERT:
 			return UpdateRect((RECT*)src, true);
 		case MRK_GODRAW:	case MRK_SSB_DRAW:
-			go = (GraphObj *) src;
-			go->DoMark(this, true);
+			go = (GraphObj *) src;				go->DoMark(this, true);
 			CurrGO = go;
 			if(CurrLabel && CurrLabel != CurrGO) {
-				HideTextCursor();
-				CurrLabel = 0L;
+				HideTextCursor();				CurrLabel = 0L;
 				}
 			return true;
 		}
@@ -428,7 +426,6 @@ anyOutput::ShowMark(void *src, int Mode)
 bool
 anyOutput::HideMark()
 {
-	CurrGO = 0L;
 	switch(MrkMode) {
 		case MRK_NONE:
 			return true;
@@ -437,13 +434,10 @@ anyOutput::HideMark()
 			return UpdateRect((RECT*)MrkRect, false);
 		case MRK_GODRAW:
 			MrkMode = MRK_NONE;					//inhibit reentrance
-			if(CurrGraph) {
-				if(CurrGraph->Command(CMD_HIDE_MARK, MrkRect, this)){
-					return true;
-					}
-				else CurrGraph->Command(CMD_REDRAW, 0L, this);
-				}
-			else ((GraphObj*)MrkRect)->DoMark(this, false);
+			if(CurrLabel && CurrLabel->Command(CMD_HIDEMARK, 0L, this))
+				CurrGraph->Command(CMD_REDRAW, 0L, this);
+			else if(MrkRect)((GraphObj*)MrkRect)->DoMark(this, false);
+			else if(CurrGraph) CurrGraph->Command(CMD_REDRAW, 0L, this);
 			return true;
 		case MRK_SSB_DRAW:
 			MrkMode = MRK_NONE;
@@ -520,10 +514,14 @@ anyOutput::TextCursor(char *txt, POINT p, POINT *fit, int *pos, int dx)
 	if(fit && pos) *pos = CurrPos;
 	disp.left = disp.right = w+dx;
 	disp.top = p.y;
+	if(TxtSet.Align & TXA_VCENTER) {
+		disp.top -= (TxtSet.iSize>>1);
+		}
 #ifdef _WINDOWS
-	disp.bottom = disp.top + TxtSet.iSize;
+	disp.bottom = disp.top + TxtSet.iSize-1;
 #else
-	disp.bottom = disp.top + iround(TxtSet.iSize*1.25);
+	disp.top -= 1;
+	disp.bottom = disp.top + iround(TxtSet.iSize*1.25)-2;
 #endif
 	ShowTextCursor(this, &disp, 0x0L);
 	return true;
