@@ -17,97 +17,90 @@
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 //
 #include "rlplot.h"
-#include <qwidget.h>
-#include <qpen.h>
-#include <qpainter.h>
-#include <qprinter.h>
-#include <qmenubar.h>
-#include <qscrollbar.h>
-#include <qdragobject.h>
+#include "menu.h"
 #include "TheDialog.h"
+#include <qwidget.h>
+#include <qapplication.h>
+#include <qnamespace.h>
+
+#if QT_VERSION < 0x040000
+	#include <qpen.h>
+	#include <qprinter.h>
+	#include <qdragobject.h>
+	#include <qmenubar.h>
+	#include <qscrollbar.h>
+	#include <qmessagebox.h>
+	#include <qpixmap.h>
+	#include <qfiledialog.h>
+	#include <qimage.h>
+	#include <qcursor.h>
+	#include <qclipboard.h>
+	#include <qbuffer.h>
+	#include <qbitmap.h>
+	#include <qtextstream.h>
+#else 
+	#include <QtGui>
+#endif
+
+#define RLP_PORT	4321		//enable clipboard server
+#ifdef RLP_PORT
+	#include <sys/socket.h>
+	#include <netdb.h>
+	#include <pthread.h>
+	#include <fcntl.h>
+	#include <unistd.h>
+	#include <sys/ioctl.h>
+	#include <linux/fs.h> 
+#endif
+
+bool ProcMenuEvent(int id, QWidget *parent, anyOutput *OutputClass, GraphObj *BaseObj);
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 class TxtCurBlink:public QObject {
 	Q_OBJECT
 public:
-	anyOutput *oCopyMark;
-	anyOutput *bmCopyMark;
-
 	TxtCurBlink();
 	void Show();
-	void showCopyMark();
+//	void showCopyMark();
 
 protected:
 	void timerEvent(QTimerEvent *);
 
 private:
-	POINT line[5];
 	bool isVis;
-	int count, cp_mark;
+	int count;
 };
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-//use sockets for to exchange clipboard data
-#ifdef RLP_PORT
+// use sockets for to exchange clipboard data
+// undefine RLP_PORT to limit clipboard to a single instance
 
-#include <qsocket.h>
-#include <qserversocket.h>
-
-class ReadCB:public QSocket {
-	Q_OBJECT
-public:
-	ReadCB(QObject *parent=0, const char *name=0, GraphObj *g=0);
-	~ReadCB();
-
-private slots:
-	void isConnected();
-	void receiveData();
-	void endData();
-	void isError(int);
-
-private:
-	int level;
-	char *format;
-	QObject *par;
-	GraphObj *DestGO;
-	QString *res;
-};
-
-class RLPsock:public QSocket {
-	Q_OBJECT
-public:
-	RLPsock(int sock, QObject *parent=0, const char *name=0, GraphObj *g=0);
-
-private slots:
-	void readClient();
-
-private:
-	QObject *par;
-	GraphObj *SourceGO;
-};
-
-class RLPserver:public QServerSocket {
-	Q_OBJECT
+class RLPserver {
 public:
 	GraphObj *SourceGO;
 
 	RLPserver(QObject* parent=0, GraphObj *g=0);
 	~RLPserver();
-	void newConnection(int socket){new RLPsock(socket, this, 0, SourceGO);};
 
 	void SetGO(GraphObj *g);
 	char *GetXML();
 	char *GetRLP();
 	char *GetTXT() {return text_plain; };
-
-signals:
-	void newConnect(RLPsock *);
+	bool ok() {return true;};
+#ifdef RLP_PORT
+	int Socket() {return sock;};
+#endif
 
 private:
 	char *text_xml, *text_rlp, *text_plain;
+#ifdef RLP_PORT
+	pthread_t thread;
+	pthread_attr_t thread_attr;
+	int sock;
+#endif
 };
 
-#endif //RLP_PORT
+#if QT_VERSION < 0x040000 
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 class RLPmenu:public QMenuBar {
@@ -125,79 +118,53 @@ private:
 
 };
 
+#else 			//Qt version >= 4.0
+
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+class RLPaction:public QAction {
+	Q_OBJECT
+public:
+	RLPaction(QWidget *par, anyOutput *o, GraphObj *g, char *name, int id);
+	~RLPaction(){;};
+
+public slots:
+	void doMenuItem();
+
+private:
+	int Id;
+	anyOutput *OutputClass;
+	QWidget *parent;
+	GraphObj *BaseObj;
+};
+
+#endif
+
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // The Qt widget class implementet for RLPlot
-class RLPwidget:public QWidget {
+class RLPwidget:public QMainWindow {
 	Q_OBJECT
 
 public:
 	QScrollBar *HScroll, *VScroll;
 	QPixmap *mempic;
+	QMenuBar *menu_bar;
+
 	RLPwidget(QWidget *par=0, const char *name=0, anyOutput *o = 0,
 		GraphObj *g = 0);
 	~RLPwidget();
+	void openHistoryFile(int idx);
 
 public slots:
 	void hScrollEvent(int pos);
 	void vScrollEvent(int pos);
-	void cmOpen();
-	void cmSaveData();
-	void cmSaveDataAs();
-	void cmExit();
-	void cmExitRLP();
-	void cmNewGraph();
-	void cmNewPage();
-	void cmDelGraph();
-	void cmAddPlot();
-	void cmAbout();
-	void cmAddRowCol();
-	void cmCopy();
-	void cmCut();
+	void cmNOP(){;};
+	void cmCopy(){ProcMenuEvent(CM_COPY, this, OutputClass, BaseObj);};
+	void cmCut(){ProcMenuEvent(CM_CUT, this, OutputClass, BaseObj);};
+	void cmZoomIn(){ProcMenuEvent(CM_ZOOMIN, this, OutputClass, BaseObj);};
+	void cmZoomOut(){ProcMenuEvent(CM_ZOOMOUT, this, OutputClass, BaseObj);};
+	void cmZoomFit(){ProcMenuEvent(CM_ZOOMFIT, this, OutputClass, BaseObj);};
 	void cmPaste();
-	void cmCopyGraph();
-	void cmSaveGraphAs();
-	void cmRedraw();
-	void cmZoom25();
-	void cmZoom50();
-	void cmZoom100();
-	void cmZoom200();
-	void cmZoom400();
-	void cmZoomIn();
-	void cmZoomOut();
-	void cmZoomFit();
-	void cmPrint();
-	void cmExport();
-	void cmDelObj();
-	void cmUpdate();
-	void cmDefaults();
-	void cmAddAxis();
-	void cmAddLegend();
-	void cmLayers();
-	void cmUndo();
-	void cmFillRange();
-	void cmInsRow();
-	void cmInsCol();
-	void cmDelRow();
-	void cmDelCol();
-	void cmtStandard();
-	void cmtDraw();
-	void cmtPolyline();
-	void cmtPolygon();
-	void cmtRectangle();
-	void cmtRoundrect();
-	void cmtEllipse();
-	void cmtArrow();
-	void cmtText();
-	void cmSmplStat();
-	void cmRepCmeans();
-	void cmRepregr();
-	void cmReptwoway();
-	void cmFile1() {openHistoryFile(0);};
-	void cmFile2() {openHistoryFile(1);};
-	void cmFile3() {openHistoryFile(2);};
-	void cmFile4() {openHistoryFile(3);};
-	void cmFile5() {openHistoryFile(4);};
-	void cmFile6() {openHistoryFile(5);};
+	void cmUndo(){if(BaseObj) BaseObj->Command(CMD_UNDO, 0L, OutputClass);};
 
 protected:
 	void paintEvent(QPaintEvent *);
@@ -214,12 +181,14 @@ private:
 	QWidget *parent;
 	anyOutput *OutputClass;
 	GraphObj *BaseObj;
-
-	void openHistoryFile(int idx);
 };
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+#if QT_VERSION < 0x040000
 class DlgWidget:public QWidget {
+#else
+class DlgWidget:public QWidget {
+#endif
 	Q_OBJECT
 public:
 	QPixmap *mempic;
@@ -248,14 +217,18 @@ private:
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 class BitMapQT:public anyOutput {
 public:
-	QWidget *widget;
+	QMainWindow *widget;
+	QWidget *dlgwidget;
 	HatchOut *hgo;
 	QPixmap *mempic;
 	QImage *image;
 	QPen qPen;
 	QPainter qPainter;
 	QFont qFont;
+	void *ShowObj;				//eph_obj
+	void *ShowAnimated;			//copy mark
 
+	BitMapQT(GraphObj *g, QMainWindow *wi, int vr = 98, int hr = 98);
 	BitMapQT(GraphObj *g, QWidget *wi, int vr = 98, int hr = 98);
 	BitMapQT(int w, int h, double hr, double vr);
 	~BitMapQT();
@@ -283,18 +256,23 @@ public:
 class OutputQT:public BitMapQT {
 public:
 	QScrollBar *HScroll, *VScroll;
+#if QT_VERSION < 0x040000
+	RLPmenu *menu;
+#else
+	QAction *itFil1, *itFil2, *itFil3, *itFil4, *itFil5, *itFil6;
+	QAction *ittStd, *ittDraw, *ittPl, *ittPg, *ittRec, *ittRrec, *ittElly, *ittArr, *ittTxt;
+#endif
 
 	OutputQT(GraphObj *g);
 	OutputQT(DlgWidget *wi);
 	~OutputQT();
 	bool ActualSize(RECT *rc);
-	void Focus(){if(widget){widget->show(); widget->setActiveWindow();widget->raise();}};
+	void Focus(){if(widget){widget->show(); widget->activateWindow();widget->raise();}};
 	void Caption(char *txt);
 	void MouseCursor(int cid, bool force);
 	bool SetScroll(bool isVert, int iMin, int iMax, int iPSize, int iPos);
 	bool EndPage();
 	bool UpdateRect(RECT *rc, bool invert);
-	void ShowBitmap(int x, int y, anyOutput* src);
 	void ShowLine(POINT * pts, int cp, DWORD color);
 	void ShowEllipse(POINT p1, POINT p2, DWORD color); 
 	bool SetMenu(int type);
@@ -304,7 +282,6 @@ public:
 
 private:
 	GraphObj *BaseObj;
-	RLPmenu *menu;
 };
 
 class PrintQT:public anyOutput{
@@ -335,8 +312,13 @@ private:
 	QPen qPen;
 	QFont qFont;
 	QPainter qPainter;
+#if QT_VERSION >= 0x040000
+	QMatrix dxf;
+#else
 	QWMatrix dxf;
+#endif
 	char *fileName;
 	GraphObj *go;
 	bool bPrinting;
 };
+
