@@ -109,7 +109,8 @@ typedef struct {
 
 class DlgRoot:public tag_DlgObj {
 public:
-	anyOutput *CurrDisp;
+	anyOutput *CurrDisp;		//the dialog's output class
+	void *hDialog;				//handle to the dialog window/widget
 
 	DlgRoot(DlgInfo *tmpl, DataObj *d);
 	~DlgRoot();
@@ -137,8 +138,8 @@ public:
 	anyOutput *GetOutputClass(){return CurrDisp;};
 
 private:
+	int res_q[256], res_put, res_get, cDlgs, cContinue;
 	anyOutput *ParentOut;
-	int cDlgs, Result, cContinue;
 	DataObj *data;
 	Dialog *oldFocus, *oldDefault, *oldTabStop;
 	bool bActive, bRedraw;
