@@ -80,43 +80,40 @@ void NiceAxis(AxisDEF *axis, int nTick)
 		}
 	if(diff <= 0.0) return;
 	logStep = log10(diff/(double)nTick);
-	Magn = floor(logStep);
-	logStep -= Magn;
-	Step = 1.0;
-	if(logStep > 0.301) Step = 2.0; 
-	if(logStep > 0.699) Step = 5.0;
-	Step *= pow(10.0, Magn);
-	HiVal = LoVal = Step * floor(axis->min/Step);
-	axis->max *=1.05f;
+	Magn = floor(logStep);			logStep -= Magn;
+	if(logStep > 0.8) Step = 10.0;
+	else if(logStep > 0.5) Step = 5.0;
+	else if(logStep > 0.2) Step = 2.0; 
+	else Step = 1.0;
+	Step *= pow(10.0, Magn);		HiVal = LoVal = Step * floor(axis->min/Step);
+	axis->max += (diff * 0.05);
 	while(HiVal < axis->max) HiVal += Step;
 	if((axis->flags & AXIS_LOG) == AXIS_LOG) {
 		if (LoVal > defs.min4log) axis->min = LoVal;
 		if ((LoVal + Step) > defs.min4log && (LoVal + Step) < axis->min) axis->min = LoVal+Step;
 		}
 	else axis->min = LoVal;
-	axis->max = HiVal;
-	axis->Start = axis->min;
-	axis->Step = Step;
+	axis->max = HiVal;		axis->Start = axis->min;		axis->Step = Step;
 }
 
 void NiceStep(AxisDEF *axis, int nTick)
 {
-	double diff, logStep, Step, Magn;
+	double diff, d, logStep, Step, Magn;
 	int i;
 
-	diff = axis->max - axis->min;
+	diff = axis->max - axis->min;		d = axis->Step != 0.0 ? diff/axis->Step : HUGE_VAL;
+	if((d - floor(d)) < 0.1 && axis->Step != 0.0 && diff/axis->Step < 12.0)return;
 	if(axis->breaks) for(i = 0; i < axis->nBreaks; i++) {
 		diff -= fabs(axis->breaks[i].fy - axis->breaks[i].fx);
 		}
-	if(diff < 0.0) return;
+	if(diff <= 0.0) return;
 	logStep = log10(diff/(double)nTick);
-	Magn = floor(logStep);
-	logStep -= Magn;
-	Step = 1.0;
-	if(logStep > 0.301) Step = 2.0; 
-	if(logStep > 0.699) Step = 5.0;
-	Step *= pow(10.0, Magn);
-	axis->Step = Step;
+	Magn = floor(logStep);				logStep -= Magn;
+	if(logStep > 0.8) Step = 10.0;
+	else if(logStep > 0.5) Step = 5.0;
+	else if(logStep > 0.2) Step = 2.0; 
+	else Step = 1.0;
+	Step *= pow(10.0, Magn);			axis->Step = Step;
 }
 
 double base4log(AxisDEF *axis, int direc)
@@ -524,6 +521,18 @@ double NiceValue(double fv)
 	return sign*pow(10.0, magn-1.0) *(double)i;
 }
 
+char *NiceTime(double val)
+{
+	rlp_datetime dt;
+
+	parse_datevalue(&dt, val);
+	if(dt.year > 1905) {
+		if(dt.hours) return date2text(&dt, defs.fmt_datetime);
+		else return date2text(&dt, defs.fmt_date);
+		}
+	else return date2text(&dt, defs.fmt_time);
+}
+
 char *Int2ColLabel(int nr1, bool uc)
 {
 	static char RetTxt[12];
@@ -1100,7 +1109,7 @@ POINT *MakeArc(int ix, int iy, int r, int qad, long *npts)
 			}
 		}
 	if(*npts < 3) return 0L;
-	if(rpts = (POINT*)realloc(pts, sizeof(POINT)*(*npts))) return rpts;
+	if(rpts = (POINT*)realloc(pts, sizeof(POINT)*(*npts+4))) return rpts;
 	return pts;
 }
 
