@@ -300,14 +300,14 @@ void Qt_Box()
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Display blinking text cursor
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-anyOutput *oTxtCur = 0L, *oCopyMark = 0L;
+static anyOutput *oTxtCur = 0L, *oCopyMark = 0L;
 RECT rTxtCur, rCopyMark;
-bool bTxtCur = false, bTxtCurIsVis = false, bSuspend = false;
-DWORD cTxtCur = 0x0L;
-HWND hwndTxtCur = 0L;
-int iTxtCurCount = 0;
-POINT ptTxtCurLine[2];
-BitMapWin *bmCopyMark = 0L;
+static bool bTxtCur = false, bTxtCurIsVis = false, bSuspend = false;
+static DWORD cTxtCur = 0x0L;
+static HWND hwndTxtCur = 0L;
+static int iTxtCurCount = 0;
+static POINT ptTxtCurLine[2];
+static BitMapWin *bmCopyMark = 0L;
 
 void HideTextCursor()
 {
@@ -340,10 +340,12 @@ void ShowTextCursor(anyOutput *out, RECT *disp, DWORD color)
 
 void HideCopyMark()
 {
+	BitMapWin *CurrCopyMark;
+
 	if(bmCopyMark && oCopyMark) {
-		bmCopyMark = 0L;
+		CurrCopyMark = bmCopyMark;		bmCopyMark = 0L;
 		oCopyMark->UpdateRect(&rCopyMark, false);
-		delete bmCopyMark;
+		delete CurrCopyMark;
 		}
 	bmCopyMark = 0L;	oCopyMark = 0L;
 }
@@ -388,8 +390,8 @@ void SuspendAnimation(anyOutput *o, bool bSusp)
 		}
 }
 
-LineDEF liCopyMark1 = {0.0f, 1.0f, 0x00ffffffL, 0L};
-LineDEF liCopyMark2 = {0.0f, 6.0f, 0x0L, 0xf0f0f0f0L};
+static LineDEF liCopyMark1 = {0.0f, 1.0f, 0x00ffffffL, 0L};
+static LineDEF liCopyMark2 = {0.0f, 6.0f, 0x0L, 0xf0f0f0f0L};
 
 LRESULT FAR PASCAL TimerWndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 {
@@ -688,6 +690,7 @@ bool com_oTextOut(int x, int y, char *atxt, int cb, HFONT *hFont, HDC *dc, TextD
 	unsigned char *utxt = (unsigned char*)atxt;
 	w_char *uc;
 	int i;
+	bool bRet;
 
 	if(!*hFont || !atxt || !atxt[0]) return false;
 	if(cb < 1) cb = (int)strlen(atxt);
@@ -699,12 +702,9 @@ bool com_oTextOut(int x, int y, char *atxt, int cb, HFONT *hFont, HDC *dc, TextD
 			else uc[i] = utxt[i];
 			}
 		}
-	else {
-		for(i = 0; utxt[i]; i++) uc[i] = utxt[i];
-		}
-	com_oTextOutW(x, y, uc, cb, hFont, dc, td, o);
-	free(uc);
-	return false;
+	else for(i = 0; utxt[i]; i++) uc[i] = utxt[i];
+	bRet = com_oTextOutW(x, y, uc, cb, hFont, dc, td, o);
+	free(uc);		return bRet;
 }
 
 bool com_SetTextSpec(TextDEF *set, anyOutput *o, HFONT *hFont,	TextDEF *TxtSet, 
@@ -1143,85 +1143,85 @@ OutputWin::Caption(char *txt)
 	SetWindowText(hWnd, txt);
 }
 
-unsigned char hand_bits[] =	{	//hand cursor bitmap
+const static unsigned char hand_bits[] =	{	//hand cursor bitmap
 	0x01, 0x80, 0x1b, 0xf0, 0x3f, 0xf8, 0x3f, 0xfa,
 	0x1f, 0xff, 0x1f, 0xff, 0x6f, 0xff, 0xff, 0xff,
 	0xff, 0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x3f, 0xfc,
 	0x1f, 0xfc, 0x0f, 0xf8, 0x07, 0xf8, 0x07, 0xf8};
 
-unsigned char hand_mask[] =	{	//hand cursor mask
+const static unsigned char hand_mask[] =	{	//hand cursor mask
 	0xff, 0xff, 0xfe, 0x7f, 0xe6, 0x4f, 0xe6, 0x4f,
 	0xf2, 0x4d, 0xf2, 0x49, 0x78, 0x09, 0x98, 0x01,
 	0x88, 0x03, 0xc0, 0x03, 0xc0, 0x03, 0xe0, 0x07,
 	0xf0, 0x07, 0xf8, 0x0f, 0xfc, 0x0f, 0xfc, 0x0f};
 
-unsigned char zoom_bits[] =	{	//zoom cursor bitmap
+const static unsigned char zoom_bits[] =	{	//zoom cursor bitmap
 	0x00, 0x00, 0x00, 0x00, 0x01, 0xa0, 0x06, 0x30,
 	0x08, 0x08, 0x10, 0x84, 0x10, 0x84, 0x20, 0x02,
 	0x26, 0x32, 0x20, 0x02, 0x10, 0x84, 0x10, 0x84,
 	0x08, 0x08, 0x06, 0x30, 0x01, 0xa0, 0x00, 0x00};
 
-unsigned char zoom_mask[] =	{	//zoom cursor mask
+const static unsigned char zoom_mask[] =	{	//zoom cursor mask
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
 
-unsigned char paste_bits[] =	{	//paste cursor bitmap
+const static unsigned char paste_bits[] =	{	//paste cursor bitmap
 	0x20, 0x00, 0x20, 0x00, 0xf8, 0x00, 0x20, 0x00,
 	0x23, 0xfe, 0x07, 0xff, 0x07, 0xff, 0x07, 0xff,
 	0x07, 0xff, 0x07, 0xff, 0x07, 0xff, 0x07, 0xff,
 	0x07, 0xff, 0x07, 0xff, 0x03, 0xfe, 0x00, 0x00};
 
-unsigned char paste_mask[] =	{	//paste cursor mask
+const static unsigned char paste_mask[] =	{	//paste cursor mask
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xfc, 0x05, 0xfd, 0x05, 0xfd, 0xf9,
 	0xfc, 0x01, 0xfc, 0x01, 0xfc, 0x01, 0xfc, 0x01,
 	0xfc, 0x01, 0xfc, 0x01, 0xff, 0xff, 0xff, 0xff};
 
-unsigned char drawpen_bits[] =	{	//draw cursor bitmap
+const static unsigned char drawpen_bits[] =	{	//draw cursor bitmap
 	0xc0, 0x00, 0xf0, 0x00, 0x7c, 0x00, 0x7f, 0x00,
 	0x3f, 0x80, 0x3f, 0xc0, 0x1f, 0xe0, 0x1f, 0xf0,
 	0x0f, 0xf8, 0x07, 0xfc, 0x03, 0xfe, 0x01, 0xff,
 	0x00, 0xff, 0x00, 0x7e, 0x00, 0x3c, 0x00, 0x18};
 
-unsigned char drawpen_mask[] =	{	//draw cursor mask
+const static unsigned char drawpen_mask[] =	{	//draw cursor mask
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf3, 0xff,
 	0xe0, 0xff, 0xe2, 0x7f, 0xf1, 0x3f, 0xf0, 0x9f,
 	0xf8, 0x4f, 0xfc, 0x27, 0xfe, 0x13, 0xff, 0x0b,
 	0xff, 0x87, 0xff, 0xcf, 0xff, 0xff, 0xff, 0xff};
 
-unsigned char drect_bits[] =	{	//draw rectangle bitmap
+const static unsigned char drect_bits[] =	{	//draw rectangle bitmap
 	0x20, 0x00, 0x20, 0x00, 0xf8, 0x00, 0x20, 0x00,
 	0x20, 0x00, 0x00, 0x00, 0x1f, 0xff, 0x1f, 0xff,
 	0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff,
 	0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x00, 0x00};
 
-unsigned char drect_mask[] =	{	//draw rectangle mask
+const static unsigned char drect_mask[] =	{	//draw rectangle mask
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0x01,
 	0xf0, 0x01, 0xf0, 0x01, 0xf0, 0x01, 0xf0, 0x01,
 	0xf0, 0x01, 0xf0, 0x01, 0xff, 0xff, 0xff, 0xff};
 
-unsigned char drrect_bits[] =	{	//draw rounded rectangle bitmap
+const static unsigned char drrect_bits[] =	{	//draw rounded rectangle bitmap
 	0x20, 0x00, 0x20, 0x00, 0xf8, 0x00, 0x20, 0x00,
 	0x20, 0x00, 0x00, 0x00, 0x07, 0xfc, 0x0f, 0xfe,
 	0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff,
 	0x1f, 0xff, 0x0f, 0xfe, 0x07, 0xfc, 0x00, 0x00};
 
-unsigned char drrect_mask[] =	{	//draw rounded rectangle mask
+const static unsigned char drrect_mask[] =	{	//draw rounded rectangle mask
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x03,
 	0xf0, 0x01, 0xf0, 0x01, 0xf0, 0x01, 0xf0, 0x01,
 	0xf0, 0x01, 0xf8, 0x03, 0xff, 0xff, 0xff, 0xff};
 
-unsigned char delly_bits[] =	{	//draw ellipse bitmap
+const static unsigned char delly_bits[] =	{	//draw ellipse bitmap
 	0x20, 0x00, 0x20, 0x00, 0xf8, 0x00, 0x20, 0x00,
 	0x20, 0x00, 0x00, 0x00, 0x01, 0xf0, 0x07, 0xfc,
 	0x0f, 0xfe, 0x1f, 0xff, 0x1f, 0xff, 0x1f, 0xff,
 	0x0f, 0xfe, 0x07, 0xfc, 0x01, 0xf0, 0x00, 0x00};
 
-unsigned char delly_mask[] =	{	//draw ellipse mask
+const static unsigned char delly_mask[] =	{	//draw ellipse mask
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x0f,
 	0xf8, 0x03, 0xf0, 0x01, 0xf0, 0x01, 0xf0, 0x01,
@@ -1426,7 +1426,7 @@ OutputWin::ShowEllipse(POINT p1, POINT p2, DWORD color)
 bool
 OutputWin::SetMenu(int type)
 {
-	HMENU hMenu = 0L;
+	HMENU hMenu = 0L, hPopup = 0L;
 
 	switch(type) {
 	case MENU_NONE:
@@ -1435,7 +1435,13 @@ OutputWin::SetMenu(int type)
 		hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(MENU_2));
 		break;
 	case MENU_GRAPH:
-		hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(MENU_1));
+/*
+		hMenu = CreateMenu();
+		hPopup = CreatePopupMenu();
+		AppendMenu(hPopup, MF_STRING, CM_OPEN, "&Open");
+		AppendMenu(hMenu, MF_POPUP, (UINT_PTR)hPopup, "&File");
+//		AppendMenu(hMenu, MF_POPUP, (unsigned int)hPopup, "&File");
+//*/		hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(MENU_1));
 		break;
 	case MENU_PAGE:
 		hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(MENU_3));
@@ -1764,20 +1770,14 @@ PrintWin::PrintWin()
 	double pw, ph;
 
 	PrintDriver = PrintDevice = PrintPort = 0L;
-	hPen = 0L;
-	hBrush = 0L;
-	hFont = 0L;
-	hDC = 0L;
-	hgo = 0L;
-	units = defs.cUnits;
-	i = j = 0;
+	hPen = 0L;		hBrush = 0L;			hFont = 0L;		hDC = 0L;
+	hgo = 0L;		units = defs.cUnits;	i = j = 0;
 	GetProfileString("windows", "device", "", TmpTxt, 4096);
 	while(TmpTxt[i] && TmpTxt[i] != ',') i++;
 	TmpTxt[i] = 0;
 	if (i >2) {
 		PrintDevice = _strdup(TmpTxt);
-		i++;
-		j = i;
+		i++;			j = i;
 		while(TmpTxt[i] && TmpTxt[i] != ',') i++;
 		if(i-j > 2) {
 			TmpTxt[i] = 0;
@@ -1810,12 +1810,9 @@ PrintWin::PrintWin()
 
 PrintWin::~PrintWin()
 {
-	if(PrintDriver) free(PrintDriver);
-	if(PrintDevice) free(PrintDevice);
-	if(PrintPort) free(PrintPort);
-	if(hPen) DeleteObject(hPen);
-	if(hBrush) DeleteObject(hBrush);
-	if(hFont) DeleteObject(hFont);
+	if(PrintDriver) free(PrintDriver);		if(PrintDevice) free(PrintDevice);
+	if(PrintPort) free(PrintPort);			if(hPen) DeleteObject(hPen);
+	if(hBrush) DeleteObject(hBrush);		if(hFont) DeleteObject(hFont);
 }
 
 bool
@@ -2227,12 +2224,16 @@ long OpenFileFromHistory(OutputWin *w, GraphObj *g, int id)
 	case 3:			name = defs.File4;			break;
 	case 4:			name = defs.File5;			break;
 	case 5:			name = defs.File6;			break;
+	default:		return 0;
 		}
-	if(name && FileExist(name)) {
+	if(name && name[0] && FileExist(name)) {
 		g->Command(CMD_DROPFILE, name, w);
 		defs.FileHistory(name);
 		w->FileHistory();
 		}
+	else {
+		ErrorBox("The selected file   \ndoes not exist!\n");
+		}
 	return 0;
 }
 
@@ -2343,6 +2344,8 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 			SetCursor(LoadCursor(NULL, IDC_ARROW));
 		return 0;
 	case WM_DESTROYCLIPBOARD:
+		if(g && w) g->Command(CMD_HIDEMARK, 0L, w);
+		HideCopyMark();
 		return 0;
 	case WM_RENDERALLFORMATS:
 		// we do not support leaving data on the clipboard after exit
@@ -2402,9 +2405,17 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 			return 0;
 		case CM_COPY:			case CM_CUT:		case CM_COPYGRAPH:
 			EmptyClip();
-			if(g->Id != GO_SPREADDATA && CurrGO && CurrGO->Id == GO_TEXTFRAME) {
-				if(CurrGO->Command(CMD_COPY, 0L, w))return 0;
-				}
+			if(CurrGO && g->Id != GO_SPREADDATA) {
+				if(CurrGO->Id == GO_POLYLINE || CurrGO->Id == GO_POLYGON || CurrGO->Id == GO_RECTANGLE
+					|| CurrGO->Id == GO_ROUNDREC || CurrGO->Id == GO_ELLIPSE || CurrGO->Id == GO_BEZIER) {
+					OpenClipboard(hwnd);
+					CopyGraph(CurrGO, cf_rlpobj, w);		copy_obj = CurrGO;
+					CloseClipboard();						return 0;
+					}
+				else if(CurrGO->Id == GO_TEXTFRAME) {
+					if(CurrGO->Command(CMD_COPY, 0L, w)) return 0;
+					}
+				}	
 			OpenClipboard(hwnd);
 			if(g->Id == GO_SPREADDATA && g->Command(wParam == CM_CUT ? CMD_CUT : CMD_QUERY_COPY, 0L, w)) {
 				SetClipboardData(CF_TEXT, NULL);
@@ -2418,6 +2429,7 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 					CopyGraph(CurrGraph, cf_rlpobj, w);		copy_obj = CurrGraph;
 					}
 				}
+			else if (wParam == CM_CUT)return 0;
 			else if(CurrGraph && CurrGraph->Id == GO_GRAPH){
 				SetClipboardData(CF_ENHMETAFILE, NULL);
 				SetClipboardData(CF_BITMAP, NULL);
@@ -2621,12 +2633,24 @@ long FAR PASCAL WndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		case CM_REPANOV:
 			if(g->data) rep_anova(g, g->data);
 			return 0;
+		case CM_REPTWANR:
+			if(g->data) rep_twoway_anova(g, g->data);
+			return 0;
 		case CM_REPKRUSKAL:
 			if(g->data) rep_kruskal(g, g->data);
 			return 0;
+		case CM_REPTWANOV:
+			if(g->data) rep_twanova(g, g->data);
+			return 0;
+		case CM_REPFRIEDM:
+			if(g->data) rep_fmanova(g, g->data);
+			return 0;
 		case CM_REPREGR:
 			if(g->data) rep_regression(g, g->data);
 			return 0;
+		case CM_ROBUSTLINE:
+			if(g->data) rep_robustline(g, g->data);
+			return 0;
 		case CM_CORRELM:
 			if(g->data) rep_correl(g, g->data, 0);
 			return 0;
@@ -2792,6 +2816,7 @@ void *CreateDlgWnd(char *title, int x, int y, int width, int height, tag_DlgObj
 	w = new OutputWin(0L, hDlg);
 	w->units = defs.cUnits;
 	if(hDlg && w && w->Erase(0x00e0e0e0L)) {
+		((DlgRoot*)d)->hDialog = hDlg;
 		SetWindowLong(hDlg, GWL_USERDATA, lptrref(w));
 		SetWindowLong(hDlg, 0, lptrref(d));
 		if(flags & 0x01) {					//center on screen
