@@ -1,4 +1,4 @@
-//exprlp.cpp, Copyright (c) 2002, 2003, 2004, 2005 R.Lackner
+//exprlp.cpp, Copyright (c) 2002-2006 R.Lackner
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -53,342 +53,6 @@ unsigned char* PasteText()
 	return 0L;
 }
 
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// STUBS: we do not need dialogs
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-bool ShowLayers(GraphObj *root)
-{
-	return false;
-}
-
-bool GetBitmapRes(double *res, double *width, double *height, char *header)
-{
-	return false;
-}
-
-bool GetPaper(double *w, double *h)
-{
-	*w = *h = 1.0;
-	return true;
-}
- 
-bool Symbol::PropertyDlg()
-{
-	return false;
-}
-
-bool Bubble::PropertyDlg()
-{
-	return false;
-}
-
-bool Bar::PropertyDlg()
-{
-	return false;
-}
-
-bool DataLine::PropertyDlg()
-{
-	return false;
-}
-
-bool DataPolygon::PropertyDlg()
-{
-	return false;
-}
-
-bool RegLine::PropertyDlg()
-{
-	return false;
-}
-
-bool SDellipse::PropertyDlg()
-{
-	return false;
-}
-
-bool ErrorBar::PropertyDlg()
-{
-	return false;
-}
-
-bool Arrow::PropertyDlg()
-{
-	return false;
-}
-
-bool Box::PropertyDlg()
-{
-	return false;
-}
-
-bool Whisker::PropertyDlg()
-{
-	return false;
-}
-
-bool DropLine::PropertyDlg()
-{
-	return false;
-}
-
-bool Sphere::PropertyDlg()
-{
-	return false;
-}
-
-bool Plane3D::PropertyDlg()
-{
-	return false;
-}
-
-bool Brick::PropertyDlg()
-{
-	return false;
-}
-
-bool DropLine3D::PropertyDlg()
-{
-	return false;
-}
-
-bool Arrow3D::PropertyDlg()
-{
-	return false;
-}
-
-bool Line3D::PropertyDlg()
-{
-	return false;
-}
-
-bool Label::PropertyDlg()
-{
-	return false;
-}
-
-bool segment::PropertyDlg()
-{
-	return false;
-}
-
-bool polyline::PropertyDlg()
-{
-	return false;
-}
-
-bool polygon::PropertyDlg()
-{
-	return false;
-}
-
-bool rectangle::PropertyDlg()
-{
-	return false;
-}
-
-bool PlotScatt::PropertyDlg()
-{
-	return false;
-}
-
-bool xyStat::PropertyDlg()
-{
-	return false;
-}
-
-bool Regression::PropertyDlg()
-{
-	return false;
-}
-
-bool FreqDist::PropertyDlg()
-{
-	return false;
-}
-
-bool BubblePlot::PropertyDlg()
-{
-	return false;
-}
-
-bool PolarPlot::PropertyDlg()
-{
-	return false;
-}
-
-bool PolarPlot::Config()
-{
-	return false;
-}
-
-bool BoxPlot::PropertyDlg()
-{
-	return false;
-}
-
-bool DensDisp::PropertyDlg()
-{
-	return false;
-}
-
-bool StackBar::PropertyDlg()
-{
-	return false;
-}
-
-bool Waterfall::PropertyDlg()
-{
-	return false;
-}
-
-bool MultiLines::PropertyDlg()
-{
-	return false;
-}
-
-bool PieChart::PropertyDlg()
-{
-	return false;
-}
-
-bool StarChart::PropertyDlg()
-{
-	return false;
-}
-
-bool Function::PropertyDlg()
-{
-	return false;
-}
-
-bool Scatt3D::PropertyDlg()
-{
-	return false;
-}
-
-bool FitFunc::PropertyDlg()
-{
-	return false;
-}
-
-bool Plot3D::PropertyDlg()
-{
-	return false;
-}
-
-bool Plot3D::AddPlot(int)
-{
-	return false;
-}
-
-bool Chart25D::PropertyDlg()
-{
-	return false;
-}
-
-bool Ribbon25D::PropertyDlg()
-{
-	return false;
-}
-
-bool Func3D::PropertyDlg()
-{
-	return false;
-}
-
-bool BubblePlot3D::PropertyDlg()
-{
-	return false;
-}
-
-bool GridLine::PropertyDlg()
-{
-	return false;
-}
-
-bool Tick::PropertyDlg()
-{
-	return false;
-}
-
-bool Axis::PropertyDlg()
-{
-	return false;
-}
-
-bool Graph::AddPlot(int family)
-{
-	return false;
-}
-
-bool Graph::PropertyDlg()
-{
-	return false;
-}
-
-bool Graph::Configure()
-{
-	return false;
-}
-
-bool Graph::AddAxis()
-{
-	return false;
-}
-
-bool Page::Configure()
-{
-	return false;
-}
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// some more STUBS .....
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-char *SaveGraphAsName(char *oldname)
-{
-	return name2;
-}
-
-char *OpenGraphName(char *oldname)
-{
-	return 0L;
-}
-
-void HideTextCursor()
-{
-	return;
-}
-
-void HideTextCursorObj(anyOutput *out)
-{
-	return;
-}
-
-void ShowTextCursor(anyOutput *out, RECT *disp, DWORD color)
-{
-	return;
-}
-
-anyOutput *NewDispClass(GraphObj *g)
-{
-	return 0L;
-}
-
-bool DelDispClass(anyOutput *w)
-{
-	return false;
-}
-
-anyOutput *NewBitmapClass(int w, int h, double hr, double vr)
-{
-	return 0L;
-}
-
-bool DelBitmapClass(anyOutput *w)
-{
-	return false;
-}
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // create a root object to handle I/O
@@ -410,7 +74,7 @@ ExpRoot::ExpRoot(char *file1, char *file2):GraphObj(0L, 0L)
 	if(file2 && strcmp("-", file2)) name2 = file2;
 	else name2 = 0L;
 	go = 0L;
-	OpenGraph(this, name1, 0L);
+	OpenGraph(this, name1, 0L, false);
 	if(bDelete && name1 && name1[0]) unlink(name1);
 }
 
