@@ -95,8 +95,12 @@
 #define CM_SMPLSTAT    650
 #define CM_REPCMEANS   651
 #define CM_REPANOV     652
-#define CM_REPKRUSKAL  653
-#define CM_REPREGR     654
-#define CM_CORRELM     655
-#define CM_CORRELT     656
-#define CM_REPTWOWAY   657
+#define CM_REPTWANOV   653
+#define CM_REPFRIEDM   654
+#define CM_REPTWANR    655
+#define CM_REPKRUSKAL  656
+#define CM_REPREGR     657
+#define CM_ROBUSTLINE  658
+#define CM_CORRELM     659
+#define CM_CORRELT     660
+#define CM_REPTWOWAY   661
