@@ -326,6 +326,39 @@ void SortFpArray(int n, lfPOINT *vals)
 		}
 }
 
+//randomize array
+double *randarr(double *v0, int n, long *seed)
+{
+	double r, *v, *v_tmp;
+	int i, j, l;
+
+	if(!(v = (double*)malloc(n *sizeof(double)))) return 0L;
+	if(!(v_tmp = (double*)memdup(v0, n *sizeof(double),0))) return 0L;
+	for(l = n, i = 0; i < n; ) {
+		r = ran2(seed);			j = (int)(r *((double)l));
+		if(j < l) {
+			v[i++] = v_tmp[j];
+			if(j < l)memcpy(v_tmp+j, v_tmp+j+1, (l-j)*sizeof(double));
+			l--;
+			}
+		}
+	return v;
+}
+
+//resample array
+double *resample(double *v0, int n, long *seed)
+{
+	double r, *v;
+	int i, j;
+
+	if(!(v = (double*)malloc(n *sizeof(double)))) return 0L;
+	for(i = 0; i < n; ) {
+		r = ran2(seed);			j = (int)(r *((double)n));
+		if(j < n) v[i++] = v0[j];
+		}
+	return v;
+}
+
 //---------------------------------------------------------------------------
 // Cubic Spline Interpolation
 // Ref.: W.H. Press, B.P. Flannery, S.A. Teukolsky, W.T. Vetterling (1989), 
@@ -635,6 +668,7 @@ double lognorm_freq(double x, double m, double s)
 //chi square distribution
 double chi_dist(double x, double df, double)
 {
+	if(x <= 0.0) return 1.0;
 	return gammq(df/2.0, x/2.0);
 }
 
@@ -672,7 +706,7 @@ double pois_dist(double x, double m, double)
 //f-distribution
 double f_dist(double f, double df1, double df2)
 {
-	return betai(df2/2.0, df1/2.0, df2/(df2+df1*f));
+	return f > 0.0 ? betai(df2/2.0, df1/2.0, df2/(df2+df1*f)): 1.0;
 }
 
 double f_freq(double x, double df1, double df2)
@@ -685,6 +719,96 @@ double f_freq(double x, double df1, double df2)
 	return exp(a-b+c+d);
 }
 
+//---------------------------------------------------------------------------
+// The Weibull distribution
+//---------------------------------------------------------------------------
+double weib_dist(double x, double shape, double scale)
+{
+	double dn=1.0, sum, term, tmp;
+
+	if(shape <= 0.0 || scale <= 0.0) return HUGE_VAL;
+	if(x <= 0.0) return 0.0;
+	term = -pow(x/scale, shape);		tmp = fabs(term);
+	if(tmp < 2.22e-16) return tmp;
+	if (tmp > 0.697) return -exp(term)+1.0;
+	x = sum = term;
+	do {				//do taylor series
+		dn += 1.0 ;		term *= x/dn;		sum += term;
+		}while (fabs(term) > fabs(sum) * 2.22e-16) ;
+	return -sum;
+}
+
+double weib_freq(double x, double shape, double scale)
+{
+	double tmp1, tmp2;
+
+	if (shape <= 0.0 || scale <= 0.0) return HUGE_VAL;
+	if (x < 0) return 0.0;
+	if(x > -HUGE_VAL && x < HUGE_VAL) {
+		if(x == 0.0 && shape < 1.0) return HUGE_VAL;
+		tmp1 = pow(x / scale, shape - 1.0);
+		tmp2 = tmp1 * (x / scale);
+		return shape * tmp1 * exp(-tmp2) / scale;
+		}
+	return HUGE_VAL;
+}
+
+//---------------------------------------------------------------------------
+// The Cauchy (Lorentz) distribution
+//---------------------------------------------------------------------------
+double cauch_dist(double x, double loc, double scale)
+{
+	double y;
+
+	if(scale < 0.0) return HUGE_VAL;
+	x = (x - loc) / scale;
+	if(x > -HUGE_VAL && x < HUGE_VAL) {
+		if (fabs(x) > 1.0) {
+			y = atan(1.0/x)/_PI;		return (x > 0) ? 1.0-y : -y;
+			} 
+		else return 0.5 + atan(x)/_PI;
+		}
+	return HUGE_VAL;
+}
+
+double cauch_freq(double x, double loc, double scale)
+{
+	double y;
+
+	if(scale < 0.0) return HUGE_VAL;
+	if(x > -HUGE_VAL && x < HUGE_VAL) {
+		y = (x - loc) / scale;
+		return 1.0 / (_PI * scale * (1.0 + y*y));
+		}
+	return HUGE_VAL;
+}
+
+//---------------------------------------------------------------------------
+// The Logistic distribution
+//---------------------------------------------------------------------------
+double logis_dist(double x, double loc, double scale)
+{
+	if(scale < 0.0) return HUGE_VAL;
+	x = exp(-(x - loc) / scale);
+	if(x > -HUGE_VAL && x < HUGE_VAL) {
+		return 1.0/(1.0 + x);
+		}
+	return HUGE_VAL;
+}
+
+double logis_freq(double x, double loc, double scale)
+{
+	double e, f;
+
+	x = fabs((x - loc) / scale);
+	if(x > -HUGE_VAL && x < HUGE_VAL) {
+		e = exp(-x);     f = 1.0 + e;	
+		return  e / (scale * f*f);
+		}
+	return HUGE_VAL;
+}
+
+//---------------------------------------------------------------------------
 // Shapiro-Wilk W test and its significance level
 // Algorithm AS 394, 1995, Appl. Statist. 44(4), 547-551
 //
@@ -899,7 +1023,10 @@ void funcd(double x, double *fn, double *df, double (*sf)(double, double, double
 	else if(sf == t_dist) *df = -2.0 * t_freq(x, df1);
 	else if(sf == f_dist) *df = -1.0 * f_freq(x, df1, df2);
 	else if(sf == lognorm_dist) *df = lognorm_freq(x, df1, df2);
-	else {
+	else if(sf == weib_dist) *df = weib_freq(x, df1, df2);
+	else if(sf == cauch_dist) *df = cauch_freq(x, df1, df2);
+	else if(sf == logis_dist) *df = logis_freq(x, df1, df2);
+	else {		//numerical differentiation
 		y1 = (sf)(x * 0.995, df1, df2);		y2 = (sf)(x * 1.005, df1, df2);
 		*df = (y2-y1)*100.0/x;
 		}
@@ -1246,7 +1373,7 @@ double d_kendall(double *x, double *y, int n, char *dest, DataObj *data, double
 
 
 //linear regression
-double d_regression(double *x, double *y, int n, char *dest, DataObj *data)
+double d_regression(double *x, double *y, int n, char *dest, DataObj *data, double *ra)
 {
 	double sx, sy, dx, dy, sxy, sxx, syy, sdy, df;
 	double res[10];		// slope, intercept, mean x, mean y, SE of slope, 
@@ -1281,6 +1408,7 @@ double d_regression(double *x, double *y, int n, char *dest, DataObj *data)
 		data->Command(CMD_UPDATE, 0L, 0L);
 		delete rD;
 		}
+	if (ra)	memcpy(ra, res, 10 * sizeof(double));
 	return n;
 }
 
@@ -1303,7 +1431,7 @@ double d_covar(double *x, double *y, int n, char *dest, DataObj *data)
 }
 
 //Mann-Whitney U Test
-double d_utest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *results)
+double d_utest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *ra)
 {
 	double *da, *ta, u1, u2, su, su1, ts, dn1 = n1, dn2 = n2;
 	double res[9];
@@ -1343,9 +1471,7 @@ double d_utest(double *x, double *y, int n1, int n2, char *dest, DataObj *data,
 		data->Command(CMD_UPDATE, 0L, 0L);
 		delete rD;
 		}
-	else if(results) {
-		for(i = 0; i < 9; i++) results[i] = res[i];
-		}
+	if (ra)	memcpy(ra, res, 9 * sizeof(double));
 	return res[8];
 }
 
@@ -1383,7 +1509,7 @@ double d_ttest(double *x, double *y, int n1, int n2, char *dest, DataObj *data,
 		data->Command(CMD_UPDATE, 0L, 0L);
 		delete rD;
 		}
-	else if(results) {
+	if(results) {
 		results[0] = mx;	results[1] = sqrt(sx/(double)(n1-1));	results[2] = n1;
 		results[3] = my;	results[4] = sqrt(sy/(double)(n2-1));	results[5] = n2;
 		results[7] = df;	df = (n1-1) + (n2-1);	results[6] = betai(df/2.0, 0.5, (df/(df+d*d)));
@@ -1393,7 +1519,7 @@ double d_ttest(double *x, double *y, int n1, int n2, char *dest, DataObj *data,
 }
 
 //t-test for paired samples
-double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data)
+double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data, double *ra)
 {
 	double sx, sy, mx, my, df, cov, sd, t, p;
 	int i, r, c;
@@ -1406,9 +1532,9 @@ double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data)
 	sd = sqrt((sx+sy-2*cov)/n);
 	t = (mx-my)/sd;					df = (n-1);
 	p = betai(0.5*df, 0.5, df/(df+t*t));
+	res[0] = mx;	res[1] = sqrt(sx);	res[5] = p;
+	res[2] = my;	res[3] = sqrt(sy);	res[4] = n;
 	if((dest) && (data) && (rD = new AccRange(dest))) {
-		res[0] = mx;	res[1] = sqrt(sx);	res[5] = p;
-		res[2] = my;	res[3] = sqrt(sy);	res[4] = n;
 		rD->GetFirst(&c, &r);
 		for(i = 0; i < 6 && rD->GetNext(&c, &r); i++) {
 			data->SetValue(r, c, res[i]);
@@ -1416,11 +1542,12 @@ double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data)
 		data->Command(CMD_UPDATE, 0L, 0L);
 		delete rD;
 		}
+	if (ra)	memcpy(ra, res, 6 * sizeof(double));
 	return p;
 }
 
 //f-test
-double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data)
+double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *ra)
 {
 	int i, r, c;
 	double sx, sy, mx, my, d, df1, df2, p;
@@ -1439,9 +1566,9 @@ double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data)
 		}
 	p = 2.0 * betai(df2/2.0, df1/2.0, df2/(df2+df1*d));
 	if(p > 1.0) p = 2.0-p;
+	res[0] = mx;	res[1] = sqrt(sx);	res[2] = n1;
+	res[3] = my;	res[4] = sqrt(sy);	res[5] = n2;
 	if((dest) && (data) && (rD = new AccRange(dest))) {
-		res[0] = mx;	res[1] = sqrt(sx);	res[2] = n1;
-		res[3] = my;	res[4] = sqrt(sy);	res[5] = n2;
 		rD->GetFirst(&c, &r);
 		for(i = 0; i < 6 && rD->GetNext(&c, &r); i++) {
 			data->SetValue(r, c, res[i]);
@@ -1449,14 +1576,121 @@ double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data)
 		data->Command(CMD_UPDATE, 0L, 0L);
 		delete rD;
 		}
+	if (ra)	memcpy(ra, res, 6 * sizeof(double));
 	return p;
 }
+//---------------------------------------------------------------------------
+// Simple one way anova
+//---------------------------------------------------------------------------
+bool do_anova1(int n, int *nv, double **vals, double **res_tab, double *gm, double **means, double **ss)
+{
+	int i, j, ntot;
+	double tmp, *csums, *css, ssa, ssw, sst, mtot, d;
+
+	if(!(csums = (double*)calloc(n+1, sizeof(double)))
+		|| !(css = (double*)calloc(n+1, sizeof(double)))) return false;
+
+	for(i = ntot = 0, mtot = 0.0, d = 1.0; i< n; i++){
+		for(j = 0, csums[i] = 0.0, tmp = 1.0; j < nv[i]; j++, d+=1.0, tmp +=1.0) {
+			mtot += (vals[i][j] - mtot)/d;		
+			csums[i] += (vals[i][j] -csums[i])/tmp;
+			}
+		ntot += nv[i];
+		}
+	for(i = 0; i < n; i++) {
+		for(j = 0, css[i] = 0.0; j < nv[i]; j++) {
+			tmp = vals[i][j] - csums[i];	css[i] += (tmp*tmp);
+			}
+		}
+	for(i = 0, ssa = ssw = sst = 0.0;  i < n; i++) {
+		tmp =(csums[i] - mtot);		ssa += (tmp*tmp) * ((double)nv[i]);
+		ssw += css[i];
+		}
+	sst = ssa + ssw;
+	res_tab[0][0] = n - 1;				res_tab[1][0] = ntot - n;
+	res_tab[2][0] = ntot -1;			res_tab[0][1] = ssa;
+	res_tab[1][1] = ssw;				res_tab[2][1] = sst;
+	res_tab[0][2] = ssa/res_tab[0][0];	res_tab[1][2] = ssw/res_tab[1][0];
+	res_tab[0][3] = res_tab[0][2]/res_tab[1][2];
+	res_tab[0][4] = f_dist(res_tab[0][3], res_tab[0][0], res_tab[1][0]);
+	if(gm) *gm = mtot;
+	if(means) *means = csums;			else free(csums);
+	if(ss) *ss = css;					else free(css);
+	return true;
+}
+
+//---------------------------------------------------------------------------
+// Bartlett's Test for homogeneity of variances
+// RR Sokal & FJ Rohlf: Biometry, 3rd ed., pp. 398 ff.
+//---------------------------------------------------------------------------
+bool bartlett(int n, int *nc, double *ss, double *chi2)
+{
+	int i, sdf, df;
+	double mss, mlss, *lnss, cf;
+
+	if(!n || !nc || !ss || !chi2) return false;
+	if(!(lnss = (double*)malloc(n * sizeof(double))))return false;
+	for(i = sdf = 0, mss = mlss = cf = 0.0; i < n; i++) {
+		sdf += (df = nc[i]-1);				lnss[i] = log(ss[i]);
+		mss += (ss[i] * ((double)df));		mlss += (lnss[i] * ((double)df)); 
+		cf += (1.0/((double)df));
+		}
+	*chi2 = ((double)sdf) * log(mss/((double)sdf)) - mlss;
+	cf -= (1.0/((double)sdf));				cf = 1.0 + cf/(3.0 * ((double)(n-1)));
+	*chi2 /= cf;
+	// P = chi_dist(*chi2, n-1, 0);
+	free(lnss);		return true;
+}
+//---------------------------------------------------------------------------
+// Leven's Test for homogeneity of variances
+//---------------------------------------------------------------------------
+bool levene(int type, int n, int *nv, double *means, double **vals, double *F, double *P)
+{
+	int i, j;
+	bool bRet = false;
+	double cm, **res_tab, **cols;
+	
+	if(!n || !nv || !means || !vals) return false;
+	//setup matrix for results
+	if((res_tab = (double**)calloc(3, sizeof(double*)))
+		&& (res_tab[0] = (double*) malloc(5*sizeof(double)))
+		&& (res_tab[1] = (double*) malloc(5*sizeof(double)))
+		&& (res_tab[2] = (double*) malloc(5*sizeof(double)))
+		&& (cols = (double**)calloc(n+1, sizeof(double*)))) bRet = true;
+	//allocate mem for data
+	for(i = 0; bRet && i<n; i++) {
+		if(!(cols[i]=(double*)malloc((nv[i]+1)*sizeof(double)))) bRet = false;
+		}
+	//data are absolute differences to mean ...
+	for(i = 0, cm = 0.0; bRet && i < n; i++) {
+		switch(type) {
+			case 1:			//use means
+				cm = means[i];								break;
+			case 2:			//use medians
+				d_quartile(nv[i], vals[i], 0L, &cm, 0L);	break;
+			}
+		for(j = 0; j < nv[i]; j++) {
+			cols[i][j] = vals[i][j] > cm ? vals[i][j] - cm : cm - vals[i][j];
+			}
+		}
+	//Levene's test statistic is based on ANOVA of the differences
+	if(bRet && (bRet = do_anova1(n, nv, cols, res_tab, 0L, 0L, 0L))){
+		if(F) *F = res_tab[0][3];				if(P) *P = res_tab[0][4];
+		}
+	//clean up
+	if(bRet) {
+		for(i = 0; i < n; i++) if(cols[i]) free(cols[i]);
+		for(i = 0; i < 3; i++) if(res_tab[i]) free(res_tab[i]);
+		free(cols);								free(res_tab);	
+		}
+	return bRet;
+}
+
 //---------------------------------------------------------------------------
 // Modules from the R-project
 //
 //---------------------------------------------------------------------------
 #define M_1_SQRT_2PI	0.398942280401432677939946059934	/* 1/sqrt(2pi) */
-#define M_LN2 log(2.0)
 /*
  *  Copyright (C) 1998       Ross Ihaka
  *  Copyright (C) 2000--2005 The R Development Core Team
@@ -1661,7 +1895,7 @@ double ptukey(double q, double rr, double cc, double df, int lower_tail, int log
 
     if (df > dlarg)	return wprob(q, rr, cc);
     f2 = df * 0.5;								// calculate leading constant
-    f2lf = ((f2 * log(df)) - (df * M_LN2)) - gammln(f2);
+    f2lf = ((f2 * log(df)) - (df * log(2.0))) - gammln(f2);
     f21 = f2 - 1.0;
     // integral is divided into unit, half-unit, quarter-unit, or eighth-unit length intervals 
 	//    depending on the value of the degrees of freedom.
@@ -1842,16 +2076,11 @@ static char *dt_day[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
 
 static char *dt_days[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
 
-typedef struct rlp_datetime {
-	int aday, year, doy, month, dom, dow, hours, minutes;
-	double seconds;
-}rlp_datetime;
-
 static bool leapyear(int year) {
 	return ((year % 4 == 0 && year % 100 != 0) || year % 400 == 0);
 }
 
-static int year2aday(int y)
+int year2aday(int y)
 {
 	int aday, y1;
 
@@ -1868,6 +2097,47 @@ static void set_dow(rlp_datetime *dt)
 	dt->dow = (dt->aday %7)+1;
 }
 
+void add_date(rlp_datetime *base, rlp_datetime *inc)
+{
+	int i, dom;
+
+	if(base) {
+		if(base->month < 1) base->month = 1;
+		if(inc) {
+			base->seconds += inc->seconds;
+			if(base->seconds >= 60.0) {
+				base->minutes++;		base->seconds -= 60.0;
+				}
+			base->minutes += inc->minutes;
+			if(base->minutes >= 60) {
+				base->hours++;			base->minutes -= 60;
+				}
+			base->hours += inc->hours;
+			if(base->hours >= 24) {
+				base->dom++;			base->hours -= 24;
+				}
+			base->year += inc->year;	base->dom += inc->dom;
+			base->month += inc->month;
+			}
+		dom = dt_monthl[base->month-1];
+		if(leapyear(base->year) && base->month == 2) dom = 29;
+		if(base->dom > dom) {
+			base->month++;			base->dom -= dom;
+			}
+		if(base->month > 12) {
+			base->year++;			base->month -= 12;
+			}
+		base->aday = year2aday(base->year);
+		for(i = base->doy = 0; i < (base->month-1); i++) {
+			dom = dt_monthl[i];
+			if(i == 1 && leapyear(base->year)) dom = 29;
+			base->doy += dom;
+			}
+		base->doy += base->dom;
+		base->aday += base->doy;	set_dow(base);
+		}
+}
+
 static int parse_date (rlp_datetime *dt, char *src, char *fmt)
 {
 	int i, j, k;
@@ -1982,7 +2252,7 @@ static int parse_date (rlp_datetime *dt, char *src, char *fmt)
 	return j;
 }
 
-static char *date2text(rlp_datetime *dt, char *fmt)
+char *date2text(rlp_datetime *dt, char *fmt)
 {
 	static char res[80];
 	int i, pos;
@@ -2142,7 +2412,7 @@ static char *date2text(rlp_datetime *dt, char *fmt)
 	return res;
 }
 
-static double date2value(rlp_datetime *dt)
+double date2value(rlp_datetime *dt)
 {
 	double res;
 
@@ -2154,7 +2424,7 @@ static double date2value(rlp_datetime *dt)
 	return res;
 }
 
-static void parse_datevalue(rlp_datetime *dt, double dv)
+void parse_datevalue(rlp_datetime *dt, double dv)
 {
 	int i, j, d;
 
