@@ -29,7 +29,6 @@ inline int iround(double a) {return a < 0.0 ?(int)(a-0.499) : (int)(a+0.499);}
 #define _SQRT2	1.4142135623730950488016887242096980785696718753769
 
 #ifdef _WINDOWS					//platform is windows
-#pragma warning( disable : 4997 )
 #include <windows.h>
 #if _MSC_VER >= 1400
 #define USE_WIN_SECURE
@@ -44,7 +43,6 @@ inline int iround(double a) {return a < 0.0 ?(int)(a-0.499) : (int)(a+0.499);}
 #define w_char wchar_t
 #define _SBINC	8				//scrollbox extra space/line
 #define _TXH	3.0				//graph text default size in mm
-#define RLP_PORT	4321		//clipboard server
 
 typedef struct tagPOINT { // pt 
     long x; 
@@ -117,14 +115,17 @@ enum {CMD_NONE, CMD_ADDCHAR, CMD_ADDCHARW, CMD_SETFOCUS, CMD_KILLFOCUS, CMD_DELE
 	CMD_SET_LINE, CMD_SAVE_SYMBOLS, CMD_SAVE_TICKS, CMD_SAVE_BARS, CMD_SAVE_BARS_CONT,
 	CMD_SAVE_ERRS, CMD_SAVE_ARROWS, CMD_SAVE_DROPLINES, CMD_SAVE_LABELS, CMD_UNLOCK, CMD_SYMTEXT_UNDO,
 	CMD_FILLRANGE, CMD_BUSY, CMD_ERROR, CMD_CLEAR_ERROR, CMD_SETPARAM, CMD_SETFUNC,
-	CMD_HIDE_MARK, CMD_LEGEND, CMD_FILENAME, CMD_LAYERS, CMD_OBJTREE, CMD_TEXTDEF,
+	CMD_LEGEND, CMD_FILENAME, CMD_LAYERS, CMD_OBJTREE, CMD_TEXTDEF,
 	CMD_HASSTACK, CMD_WRITE_GRAPHS, CMD_SETFONT, CMD_SETSTYLE, CMD_COPY, CMD_PASTE,
 	CMD_INSROW, CMD_INSCOL, CMD_DELROW, CMD_DELCOL, CMD_ADDTXT, CMD_ETRACC, CMD_SHPGUP, 
 	CMD_SHPGDOWN, CMD_ERRDESC, CMD_SAVEDATA, CMD_GETMARK, CMD_PASTE_OBJ, CMD_COL_MOUSE,
-	CMD_MARKOBJ, CMD_SCALE};
+	CMD_MARKOBJ, CMD_SCALE, CMD_GETFILENAME, CMD_TEXTTHERE, CMD_HIDEMARK};
 enum {SYM_CIRCLE, SYM_CIRCLEF, SYM_RECT, SYM_RECTF, SYM_TRIAU, SYM_TRIAUF,
 	SYM_TRIAD, SYM_TRIADF, SYM_DIAMOND, SYM_DIAMONDF, SYM_PLUS, SYM_CROSS,
-	SYM_STAR, SYM_HLINE, SYM_VLINE, SYM_TEXT, SYM_POS_PARENT = 0x1000};
+	SYM_STAR, SYM_HLINE, SYM_VLINE, SYM_CIRCLEC, SYM_RECTC, SYM_TRIAUC, SYM_TRIAUL, SYM_TRIAUR,
+	SYM_TRIADC, SYM_TRIADL, SYM_TRIADR, SYM_DIAMONDC, SYM_4STAR, SYM_4STARF, SYM_5GON, SYM_5GONF, 
+	SYM_5GONC, SYM_5STAR, SYM_5STARF, SYM_6STAR, SYM_6STARF, SYM_1QUAD, SYM_2QUAD, SYM_3QUAD,
+	SYM_TEXT = 0x004f, SYM_POS_PARENT = 0x1000};
 //types of graphic objects: stored in Id and used for proper destruction of objects
 //  and retrieving information.
 enum {GO_UNKNOWN, GO_AXIS, GO_TICK, GO_GRIDLINE, GO_SYMBOL, GO_BUBBLE, GO_BAR, 
@@ -237,6 +238,11 @@ typedef struct {
 	double scale;				//zoom factor
 	}ZoomDEF;
 
+typedef struct rlp_datetime {
+	int aday, year, doy, month, dom, dow, hours, minutes;
+	double seconds;
+}rlp_datetime;
+
 // Axis definitions are stored in the following structure
 // not to be confused with the Axis class grapic object
 // bits stored in flags havethe following meaning
@@ -259,6 +265,7 @@ typedef struct {
 #define AXIS_LOG          0x1000
 #define AXIS_RECI         0x2000
 #define AXIS_SQR          0x3000
+#define AXIS_DATETIME     0x4000		// its a date- or time axis
 #define AXIS_X_DATA       0x10000		// loc.x is data coordinates
 #define AXIS_Y_DATA       0x20000       // loc.y is data coordinates
 #define AXIS_Z_DATA       0x40000       // loc.z is data coordinates
@@ -539,6 +546,11 @@ typedef struct _fmt_txt_info {
 	char *txt;
 }fmt_txt_info;
 
+typedef struct _fmt_uc_info {
+	int tag, cb;
+	w_char *uc_txt;
+}fmt_uc_info;
+
 public:
 	fmtText();
 	fmtText(anyOutput *o, int x, int y, char *txt);
@@ -559,13 +571,15 @@ private:
 	bool SetTextDef(TextDEF *td, int idx);
 	bool Parse();
 	void DrawBullet(anyOutput *o, int x, int y, int type, double size, DWORD lc, DWORD fc);
+	bool DrawFormattedW(anyOutput *o);
 	void DrawFormatted(anyOutput *o);
 
 	char *src;
 	POINT pos;
-	int n_split;
+	int n_split, n_split_W, uc_state;
 	DWORD flags;
 	fmt_txt_info *split_text;
+	fmt_uc_info *split_text_W;
 };
 
 class TextValue {
@@ -1054,6 +1068,8 @@ public:
 	void Track(POINT *p, anyOutput *o);
 
 private:
+	anyOutput *mo;
+	RECT mrc;
 	dragHandle *dh1, *dh2;
 	POINT pts[5];
 	lfPOINT pos1, pos2;
@@ -1399,6 +1415,7 @@ public:
 	void CalcCursorPos(int x, int y, anyOutput *o);
 	void SetModified();
 	void DoPlotText(anyOutput *o);
+	bool CheckMark();
 	TextDEF *GetTextDef(){return &TextDef;};
 
 private:
@@ -1407,15 +1424,14 @@ private:
 	DWORD flags, bgcolor;
 	TextDEF TextDef;
 	LineDEF bgLine;
-	int ix, iy, CursorPos;
+	int ix, iy, m1, m2, CursorPos;
 	bool is3D;
-	RECT Cursor;
+	RECT Cursor, rm1, rm2;
 	POINT pts[5];
 	POINT *ssRef;
 	long cssRef;
 	anyOutput *defDisp;
 	bool bModified, bBGvalid;
-	fmtText *fmt_txt;
 };
 
 class mLabel:public GraphObj{
@@ -1478,7 +1494,7 @@ private:
 	void text2lines(anyOutput *o);
 	void lines2text();
 	void ShowCursor(anyOutput *o);
-	void AddChar(anyOutput *o, unsigned char c);
+	void AddChar(anyOutput *o, int c);
 	void DelChar(anyOutput *o);
 	void CalcCursorPos(int x, int y, anyOutput *o);
 	void ReplMark(anyOutput *o, char *ntext);
@@ -1676,6 +1692,7 @@ public:
 	int use_xaxis, use_yaxis, use_zaxis;		//this plot uses its own axes
 	lfPOINT xBounds, yBounds, zBounds;	//like Bounds but in 3D space
 	int hidden;						//plot (layer) is not visible
+	int x_dtype, y_dtype, z_dtype;	//data types
 	char *x_info, *y_info, *z_info;	//descriptor used e.g. for axis label or legend
 	char *data_desc;				//descriptor for data, used for legend etc
 	TextValue *x_tv, *y_tv;			//TextValue object for ordinal axes
@@ -1710,7 +1727,7 @@ public:
 	Label **Labels;
 
 	PlotScatt(GraphObj *par, DataObj *d, DWORD presel);
-	PlotScatt(GraphObj *par, DataObj *d, int cBars, Bar **bars);
+	PlotScatt(GraphObj *par, DataObj *d, int cBars, Bar **bars, ErrorBar **errs);
 	PlotScatt(GraphObj *par, DataObj *d, int nPts, Symbol **sym, DataLine *lin);
 	PlotScatt(int src);
 	~PlotScatt();
@@ -1761,6 +1778,7 @@ class FreqDist:public Plot {
 public:
 	FreqDist(GraphObj *par, DataObj *d);
 	FreqDist(GraphObj *par, DataObj *d, char* range, bool bOnce);
+	FreqDist(GraphObj *par, DataObj *d, double *vals, int nvals, int nclasses);
 	FreqDist(int src);
 	~FreqDist();
 	void DoPlot(anyOutput *o);
@@ -1938,8 +1956,11 @@ public:
 
 class GroupBars:public StackBar{
 public:
-	GroupBars(GraphObj *par, DataObj *d):StackBar(par, d){};
+	GroupBars(GraphObj *par, DataObj *d, int type):StackBar(par, d){mode = type;};
 	bool PropertyDlg();
+
+private:
+	int mode;
 };
 
 class Waterfall:public StackBar{
@@ -2283,6 +2304,7 @@ private:
 	RECT mrc;
 
 	void SetTick(long idx, double val, DWORD flags, char *txt);
+	void mkTimeAxis();
 	void CreateTicks();
 	void ManuTicks(double sa, double st, int n, DWORD flags);
 	void UpdateTicks();
@@ -2420,7 +2442,7 @@ public:
 	Axis **Axes;
 	char *filename;
 
-	Graph(GraphObj *par, DataObj *d, anyOutput *o);
+	Graph(GraphObj *par, DataObj *d, anyOutput *o, int style);
 	Graph(int src);
 	~Graph();
 	double GetSize(int select);
@@ -2441,7 +2463,7 @@ private:
 	DWORD ColDR, ColGR, ColGRL;
 	AxisDEF x_axis, y_axis;
 	FrmRect *frm_g, *frm_d;
-	bool dirty;
+	bool dirty, bDialogOpen;
 	POINT *tl_pts;
 	long tl_nPts;
 	ZoomDEF *zoom_def;
@@ -2731,7 +2753,7 @@ bool ShowLayers(GraphObj *root);
 void GetNewFill(FillDEF *oldfill);
 void ShowBanner(bool show);
 void RLPlotInfo();
-bool DoSpShSize(DataObj *dt);
+bool DoSpShSize(DataObj *dt, GraphObj *parent);
 bool FillSsRange(DataObj *d, char **range, GraphObj *msg_go);
 bool GetBitmapRes(double *res, double *width, double *height, char *header);
 void OD_scheme(int, void *, RECT *, anyOutput *, void *, int);
@@ -2770,6 +2792,7 @@ void WriteNatFloatToBuff(char *buff, double val);
 bool Txt2Flt(char *txt, double *val);
 void RmTrail(char *txt);
 double NiceValue(double fv);
+char *NiceTime(double val);
 char *Int2ColLabel(int nr, bool uc);
 char *mkCellRef(int row, int col);
 char *mkRangeRef(int r1, int c1, int r2, int c2);
@@ -2840,6 +2863,7 @@ void DoExportEps(GraphObj *g, char *FileName, DWORD flags);
 void DoExportTif(GraphObj *g, char *FileName, DWORD flags);
 
 //prototypes mfcalc.cpp
+void LockData(bool lockExec, bool lockWrite);
 char  *yywarn(char *txt, bool bNew);
 bool do_xyfunc(DataObj *, double, double, double, char *, lfPOINT **, long *, char *);
 bool do_func3D(DataObj *d, double x1, double x2, double xstep, double z1, double z2, double zstep, 
@@ -2858,6 +2882,8 @@ bool Check_MRQerror();
 void SortArray(int n, double *vals);
 void SortArray2(int n, double *a1, double *a2);
 void SortFpArray(int n, lfPOINT *vals);
+double *randarr(double *v0, int n, long *seed);
+double *resample(double *v0, int n, long *seed);
 void spline(lfPOINT *v, int n, double *y2);
 double gammln(double x);
 double factrl(int n);
@@ -2883,6 +2909,12 @@ double t_freq(double t, double df);
 double pois_dist(double x, double m, double);
 double f_dist(double f, double df1, double df2);
 double f_freq(double f, double df1, double df2);
+double weib_dist(double x, double shape, double scale);
+double weib_freq(double x, double shape, double scale);
+double cauch_dist(double x, double loc, double scale);
+double cauch_freq(double x, double loc, double scale);
+double logis_dist(double x, double loc, double scale);
+double logis_freq(double x, double loc, double scale);
 double ks_dist(int n, double d);
 void swilk1(int n, double *v0, double (*func)(double, double, double), double p1, double p2, 
 	bool bsorted, double *w, double *p);
@@ -2902,15 +2934,23 @@ double d_rank(int n, double *v, double v1);
 void crank(int n, double *w0, double *s);
 double d_spearman(double *x, double *y, int n, char *dest, DataObj *data, double *ra);
 double d_kendall(double *x, double *y, int n, char *dest, DataObj *data, double *ra);
-double d_regression(double *x, double *y, int n, char *dest, DataObj *data);
+double d_regression(double *x, double *y, int n, char *dest, DataObj *data, double *ra);
 double d_covar(double *x, double *y, int n, char *dest, DataObj *data);
 double d_utest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *results);
 double d_ttest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *results);
-double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data);
-double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data);
+double d_ttest2(double *x, double *y, int n, char *dest, DataObj *data, double *results);
+double d_ftest(double *x, double *y, int n1, int n2, char *dest, DataObj *data, double *results);
+bool do_anova1(int n, int *nv, double **vals, double **res_tab, double *gm, double **means, double **ss);
+bool bartlett(int n, int *nc, double *ss, double *chi2);
+bool levene(int type, int n, int *nv, double *means, double **vals, double *F, double *P);
 double wprob(double w, double rr, double cc);
 double ptukey(double q, double rr, double cc, double df, int lower_tail, int log_p);
 double qtukey(double p, double rr, double cc, double df, int lower_tail, int log_p);
+int year2aday(int y);
+void add_date(rlp_datetime *base, rlp_datetime *inc);
+char *date2text(rlp_datetime *dt, char *fmt);
+double date2value(rlp_datetime *dt);
+void parse_datevalue(rlp_datetime *dt, double dv);
 bool date_value(char *desc, char *fmt, double *value);
 char *value_date(double dv, char *fmt);
 double now_today();
@@ -2920,9 +2960,13 @@ Ribbon *SurfTria(GraphObj *parent, DataObj *data, char *text1, char *text2, char
 
 //prototypes reports.cpp
 void rep_anova(GraphObj *parent, DataObj *data);
+void rep_twanova(GraphObj *parent, DataObj *data);
+void rep_fmanova(GraphObj *parent, DataObj *data);
+void rep_twoway_anova(GraphObj *parent, DataObj *data);
 void rep_kruskal(GraphObj *parent, DataObj *data);
 void rep_samplestats(GraphObj *parent, DataObj *data);
 void rep_regression(GraphObj *parent, DataObj *data);
+void rep_robustline(GraphObj *parent, DataObj *data);
 void rep_twowaytable(GraphObj *parent, DataObj *data);
 void rep_compmeans(GraphObj *parent, DataObj *data);
 void rep_correl(GraphObj *parent, DataObj *data, int style);
