@@ -160,11 +160,7 @@ SpreadWin::SpreadWin(GraphObj *par, DataObj *Data):GraphObj(par, Data)
 		w->hasHistMenu = true;
 		ssText.RotBL = ssText.RotCHAR = 0.0;
 		ssText.fSize = 0.0f;
-#ifdef _WINDOWS
 		ssText.iSize = w->un2iy(defs.GetSize(SIZE_CELLTEXT));
-#else
-		ssText.iSize = w->un2iy(defs.GetSize(SIZE_CELLTEXT)*.7);
-#endif
 		ssText.Align = TXA_VCENTER | TXA_HLEFT;		ssText.Mode = TXM_TRANSPARENT;
 		ssText.Style = TXS_NORMAL;					ssText.ColBg = 0x00e8e8e8L;
 		ssText.ColTxt = 0x00000000L;				ssText.text = 0L;
@@ -286,7 +282,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 			NumGraphs = j;	g[j-1]->DoPlot(0L);
 			return true;
 		case CMD_NEWGRAPH:
-			if((g2 = new Graph(this, d, 0L)) && g2->PropertyDlg() && 
+			if((g2 = new Graph(this, d, 0L, 0)) && g2->PropertyDlg() && 
 				Command(CMD_DROP_GRAPH, g2, o))return Command(CMD_REDRAW, 0L, o);
 			else if(g2) DeleteGO(g2);
 			Undo.SetDisp(w);
@@ -342,6 +338,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 				filename = (char*)memdup(tmpl, (int)strlen((char*)tmpl)+1, 0);
 				return Command(CMD_SETSCROLL, 0L, w);
 				}
+			else ErrorBox("The selected file is not valid\nor not accessible!\n");
 			return false;
 		case CMD_OPEN:
 			if(!Command(CMD_CAN_CLOSE, 0L, o)) return false;
@@ -357,7 +354,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 				}
 			return false;
 		case CMD_ADDROWCOL:
-			if(DoSpShSize(d)) DoPlot(o);
+			if(DoSpShSize(d, this)) DoPlot(o);
 			return true;
 		case CMD_COL_MOUSE:
 			if(o && cButtons && rButtons && (mev = (MouseEvent*)tmpl) && mev->y > o->MenuHeight) {
@@ -448,7 +445,7 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 		case CMD_TOOLMODE:		case CMD_FILLRANGE:		case CMD_CUT:
 		case CMD_PASTE_XML:		case CMD_DELROW:		case CMD_INSROW:
 		case CMD_INSCOL:		case CMD_DELCOL:		case CMD_UNDO:
-		case CMD_SHPGUP:		case CMD_SHPGDOWN:
+		case CMD_SHPGUP:		case CMD_SHPGDOWN:		case CMD_HIDEMARK:
 			bDoColWidth = false;
 			return d->Command(cmd, tmpl, o);
 		case CMD_REDRAW:
@@ -460,9 +457,10 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 		case CMD_SETSCROLL:
 			HideTextCursor();						if(!(o->ActualSize(&currRC)))return false;
 			k = (currRC.bottom-currRC.top)/d->ri->GetHeight(-1);
-			d->GetSize(&i, &j);
+			d->GetSize(&i, &j);			o->MouseCursor(MC_WAIT, true);
 			o->SetScroll(true, 0, j, k, ssOrg.y);	k = (currRC.right-currRC.left)/d->ri->GetWidth(-1);
 			o->SetScroll(false, 0, i, k, ssOrg.x);	DoPlot(o);
+			o->MouseCursor(MC_ARROW, true);
 			return true;
 		case CMD_PAGEUP:		case CMD_PAGEDOWN:
 			k = (currRC.bottom-currRC.top)/d->ri->GetHeight(-1);
@@ -489,7 +487,10 @@ SpreadWin::Command(int cmd, void *tmpl, anyOutput *o)
 		case CMD_KILLFOCUS:
 			return true;
 		case CMD_TEXTSIZE:
-			if(tmpl)ssText.iSize = *((int*)tmpl);
+			if(tmpl && *((int*)tmpl) != ssText.iSize) {
+				Undo.ValInt(this, &ssText.iSize, UNDO_CONTINUE);
+				ssText.iSize = o->un2iy(defs.GetSize(SIZE_CELLTEXT));
+				}
 			return true;
 		case CMD_CONFIG:
 			if(defs.PropertyDlg()) return Command(CMD_REDRAW, 0L, o);
@@ -616,7 +617,7 @@ bool
 SpreadWin::PrintData(anyOutput *o)
 {
 	int i, j, k, l, pgw, pfw, pcw, pch, rpp, cpp, nc, nr, ix, iy, cpages;
-	int row, col, width, height, ipad;
+	int row, col, width, height, ipad, mode;
 	double scale;
 	RECT rc, margin, margin_first;
 	POINT pp_pos, ss_pos, grid[3];
@@ -651,7 +652,7 @@ SpreadWin::PrintData(anyOutput *o)
 	tdp.Mode = TXM_TRANSPARENT;
 	tdp.Style = TXS_NORMAL;				tdp.Font = FONT_HELVETICA;	tdp.text = 0L;
 	memcpy(&td, &ssText, sizeof(TextDEF));	
-	td.Align = TXA_HCENTER | TXA_VCENTER;	
+	td.Align = TXA_HCENTER | TXA_VCENTER;
 	td.Style = TXS_NORMAL;	td.iSize = 0;
 #ifdef _WINDOWS
 	tdp.iSize = iround(o->hres/6.0);
@@ -666,6 +667,8 @@ SpreadWin::PrintData(anyOutput *o)
 	pp_pos.x = margin.left;		pp_pos.y = margin.top;
 	ss_pos.x = 0;				ss_pos.y = 0;		cpages = 1;
 	rpp = (rc.bottom - margin.top - margin.bottom - pch)/pch;
+	mode = 0;
+	if((nr * pch) < ((rc.bottom - rc.top - margin.top - margin.bottom)>>1)) mode = 1;
 	do {
 		k = (rc.right - margin.left - margin.right - pfw);
 		for(i = pgw = 0; pgw < k && (i+ss_pos.x) < nc; i++ ){
@@ -674,10 +677,12 @@ SpreadWin::PrintData(anyOutput *o)
 			}
 		if(pgw < k) {
 			cpp = i+1;
+			if(!mode && !ss_pos.x && pgw < (k>>1)) mode = 2;
 			}
 		else {
 			cpp = i-1;		pgw -= pcw;
 			}
+		mode = mode;
 		pp_pos.x = margin.left;		pp_pos.y = margin.top;
 		k = (ss_pos.x + cpp) > nc ? nc-ss_pos.x : cpp;
 		l = (ss_pos.y + rpp) > nr ? nr-ss_pos.y : rpp;
@@ -763,11 +768,11 @@ SpreadWin::PrintData(anyOutput *o)
 		if(ss_pos.y < nr) {
 			ix = pfw + pgw + iround(o->hres/3.5);
 			iy = (l+2)*pch;
-			if((margin.left + ix + pfw + pgw) < (rc.right-margin.right)) {
+			if(mode == 2 && (margin.left + ix + pfw + pgw) < (rc.right-margin.right)) {
 				margin.left += pfw + k*pcw + iround(o->hres/3.5);
 				bContinue = true;
 				}
-			else if((margin.top + iy + pch + l*pch) < (rc.bottom - margin.bottom)) {
+			else if(mode == 1 && (margin.top + iy + pch + l*pch) < (rc.bottom - margin.bottom)) {
 				margin.top += iy;	margin.left = margin_first.left;
 				bContinue = true;
 				}
@@ -929,7 +934,11 @@ SpreadData::Init(int nRows, int nCols)
 		w = Disp->w;
 		if(w) {
 			CellWidth = w->un2ix(defs.GetSize(SIZE_CELLWIDTH));
+#ifdef _WINDOWS
 			CellHeight = w->un2iy(defs.GetSize(SIZE_CELLTEXT)/defs.ss_txt) + 2;
+#else
+			CellHeight = w->un2iy(defs.GetSize(SIZE_CELLTEXT)/(defs.ss_txt *0.7)) + 2;
+#endif
 			if(CellHeight < 12)CellHeight = 19;		if(CellWidth < 40)CellWidth = 76;
 			FirstWidth = 32;						w->GetSize(&rc);
 			r_disp = (rc.bottom-rc.top)/CellHeight;
@@ -1304,6 +1313,9 @@ SpreadData::ChangeSize(int nCols, int nRows, bool bUndo)
 	bool RetVal = true;
 	
 	if(nCols == cCols && nRows == cRows) return true;
+	if(c_range){
+		free(c_range);	c_range = 0L;	EmptyClip();
+		}
 	if(bUndo) Undo.DataObject(Disp, w, this, 0L, 0L);
 	if(nRows && nRows < cRows) {
 		for (i = nRows; i < cRows; i++) {
@@ -1352,6 +1364,9 @@ SpreadData::DeleteCols()
 		InfoBox("No columns selected!");
 		return false;
 		}
+	if(c_range){
+		free(c_range);	c_range = 0L;	EmptyClip();
+		}
 	Undo.DataObject(Disp, w, this, 0L, 0L);
 	Undo.String(0L, &m_range, UNDO_CONTINUE);
 	if(!(ar = new AccRange(m_range))) return false;
@@ -1429,6 +1444,9 @@ SpreadData::InsertCols()
 		InfoBox("No columns selected!");
 		return false;
 		}
+	if(c_range){
+		free(c_range);	c_range = 0L;	EmptyClip();
+		}
 	Undo.DataObject(Disp, w, this, 0L, 0L);
 	Undo.String(0L, &m_range, UNDO_CONTINUE);
 	if(!(ar = new AccRange(m_range))) return false;
@@ -1498,6 +1516,9 @@ SpreadData::DeleteRows()
 		InfoBox("No rows selected!");
 		return false;
 		}
+	if(c_range){
+		free(c_range);	c_range = 0L;	EmptyClip();
+		}
 	Undo.DataObject(Disp, w, this, 0L, 0L);
 	Undo.String(0L, &m_range, UNDO_CONTINUE);
 	if(!(ar = new AccRange(m_range))) return false;
@@ -1575,6 +1596,9 @@ SpreadData::InsertRows()
 		InfoBox("No rows selected!");
 		return false;
 		}
+	if(c_range){
+		free(c_range);	c_range = 0L;	EmptyClip();
+		}
 	Undo.DataObject(Disp, w, this, 0L, 0L);
 	Undo.String(0L, &m_range, UNDO_CONTINUE);
 	if(!(ar = new AccRange(m_range))) return false;
@@ -1658,6 +1682,7 @@ SpreadData::DoPlot(anyOutput *o)
 	if(j >= (rc.right-rc.left))c_disp--;
 	Disp->ShowGrid(CellWidth, CellHeight, FirstWidth, &currpos);
 	rc.top = w->MenuHeight;					rc.bottom = rc.top + CellHeight;
+	LockData(false, false);
 	for(i = 0; i <= (cRows - Disp->ssOrg.y) && i <= r_disp; i++) {
 		rc.left = ri->GetFirstWidth();		rc.right = rc.left + ri->GetWidth(Disp->ssOrg.x);
 		rc.top += CellHeight;				rc.bottom += CellHeight;
@@ -1694,6 +1719,7 @@ SpreadData::DoPlot(anyOutput *o)
 			}
 		delete (ar);
 		}
+	LockData(false, false);
 	if(c_range) InitCopy(0, 0L, w);			//move animated rectangle
 	if(!(w->ActualSize(&rc))) return;		
 	rc.bottom += CellHeight;		w->UpdateRect(&rc, false);
@@ -1961,6 +1987,18 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 		if(PasteRange(cmd, (char*)tmpl)) 
 			return Disp->Command(CMD_SETSCROLL, 0L, w);
 		return false;
+	case CMD_HIDEMARK:
+		if(c_range){
+			free(c_range);	c_range = 0L;	return true;
+			}
+		return false;
+	case CMD_GETFILENAME:
+		if(!tmpl) return false;
+		if(rlw_file && rlw_file[0]) {
+			if(rlp_strcpy((char*)tmpl, TMP_TXT_SIZE, rlw_file))return true;
+			else return false;
+			}
+		return false;
 	case CMD_ETRACC:
 		if(et_racc && tmpl && ((EditText*)tmpl)->parent != this) HideMark(false);
 		et_racc = (EditText*) tmpl;
@@ -2181,9 +2219,10 @@ SpreadData::Command(int cmd, void *tmpl, anyOutput *o)
 			}
 		break;
 	case CMD_SET_CELLDIMS:
-		if(tmpl) {
-			FirstWidth = ((int*)tmpl)[0];	CellWidth = ((int*)tmpl)[1];
-			CellHeight = ((int*)tmpl)[2];
+		if(tmpl && (FirstWidth != ((int*)tmpl)[0] || CellWidth != ((int*)tmpl)[1] || CellHeight != ((int*)tmpl)[2])) {
+			Undo.ValInt(Disp, &FirstWidth, UNDO_CONTINUE);	Undo.ValInt(Disp, &CellWidth, UNDO_CONTINUE);
+			Undo.ValInt(Disp, &CellHeight, UNDO_CONTINUE);	FirstWidth = ((int*)tmpl)[0];
+			CellWidth = ((int*)tmpl)[1];					CellHeight = ((int*)tmpl)[2];
 			}
 		break;
 	case CMD_TEXTSIZE:
