@@ -1,4 +1,4 @@
-//use_gui.cpp, Copyright 2000-2006 R.Lackner
+//use_gui.cpp, Copyright 2000-2007 R.Lackner
 //
 //    This file is part of RLPlot.
 //
@@ -33,6 +33,7 @@ extern GraphObj *CurrGO, *TrackGO;			//Selected Graphic Objects
 extern Graph *CurrGraph;
 extern dragHandle *CurrHandle;
 extern UndoObj Undo;
+extern fmtText DrawFmtText;
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // Spread sheet buttons used for rows and columns
@@ -772,6 +773,7 @@ FrmRect::DoMark(anyOutput *o, bool mark)
 		if(maxRC) drag->Command(CMD_MAXRC, maxRC, o);
 		}
 	if(mark && drag){
+		if(mo) DelBitmapClass(mo);					mo = 0L;
 		memcpy(&mrc, &rDims, sizeof(RECT));
 		IncrementMinMaxRect(&mrc, 6);
 		mo = GetRectBitmap(&mrc, o);
@@ -821,7 +823,7 @@ FrmRect::Command(int cmd, void *tmpl, anyOutput *o)
 		case MOUSE_LBUP:
 			if(IsInRect(&rDims, mev->x, mev->y) && !(CurrGO) && (o)){
 				o->ShowMark(this, MRK_GODRAW);
-				if(parent && parent->Id == GO_GRAPH) CurrGraph = (Graph*)parent;
+				if(!CurrGraph && parent && parent->Id == GO_GRAPH) CurrGraph = (Graph*)parent;
 				return true;
 				}
 			}
@@ -945,8 +947,18 @@ Label::AddChar(int ci, anyOutput *o)
 	char c, *txt1 = 0L;
 	int i, j, k;
 	GraphObj *golist[2];
+	DWORD flags;
 
-	if(!o) return false;
+	if(!o || (ci != 13 && ci < 32)) return false;
+	if(CheckMark()) {
+		Undo.String(this, &TextDef.text, 0L);
+		Undo.ValInt(this, &m1, UNDO_CONTINUE);
+		Undo.ValInt(this, &m2, UNDO_CONTINUE);
+		rlp_strcpy(TextDef.text+m1, (int)strlen(TextDef.text+m1)+1, TextDef.text+m2);
+		CursorPos = m1;		flags = UNDO_CONTINUE;
+		}
+	else flags = 0L;
+	m1 = m2 = -1;
 	if(ci == 13 && parent){		//CR
 		if(parent->Id == GO_MLABEL){
 			parent->Command(CMD_SETFOCUS, this, o);
@@ -965,8 +977,8 @@ Label::AddChar(int ci, anyOutput *o)
 	if(TextDef.text && TextDef.text[0]) i = (int)strlen(TextDef.text);
 	else i = 0;		if(CursorPos > i)CursorPos = i;
 	if(ci > 254) {
-		Undo.String(this, &TextDef.text, 0L);
-		Undo.ValInt(this, &CursorPos, UNDO_CONTINUE);
+		Undo.String(this, &TextDef.text, flags);
+		Undo.ValInt(this, &CursorPos, flags = UNDO_CONTINUE);
 		txt1 = (char*)malloc((i+12)*sizeof(char));
 		if(txt1) {
 			if(j=CursorPos) k = rlp_strcpy(txt1, CursorPos+1, TextDef.text);
@@ -986,8 +998,8 @@ Label::AddChar(int ci, anyOutput *o)
 	else if( ci > 31) c = (char)ci;
 	else return false;
 	if(!TextDef.text || CursorPos < 10 || c == ' ') {
-		Undo.String(this, &TextDef.text, 0L);
-		Undo.ValInt(this, &CursorPos, UNDO_CONTINUE);
+		Undo.String(this, &TextDef.text, flags);
+		Undo.ValInt(this, &CursorPos, flags = UNDO_CONTINUE);
 		}
 	txt1 = (char*)malloc((i+4)* sizeof(char));
 	if(txt1) {
@@ -1012,8 +1024,8 @@ Label::CalcCursorPos(int x, int y, anyOutput *o)
 	x1 = ((pts[3].x + pts[4].x)>>1);	y1 = ((pts[3].y + pts[4].y)>>1);
 	ix = ((j=(x1-x))*j);	ix += ((j=(y1-y))*j);	ix = isqr(ix);
 	for(i = 1,  x1 = 0; TextDef.text[i-1]; i++) {
-		if(fmt_txt)fmt_txt->oGetTextExtent(o, &x2, &h,i);
-		else o->oGetTextExtent(TextDef.text, i, &x2, &h);
+		DrawFmtText.SetText(0L, TextDef.text, 0L, 0L);
+		DrawFmtText.oGetTextExtent(o, &x2, &h, i);
 		if(x1 <= ix && x2 >= ix) {
 			if((ix-x1) < (x2-ix)) CursorPos = i-1;
 			else CursorPos = i;
@@ -1096,6 +1108,7 @@ Axis::DoMark(anyOutput *o, bool mark)
 
 	if(axis->flags & AXIS_ANGULAR) {
 		if(mark) {
+			if(mo) DelBitmapClass(mo);					mo = 0L;
 			o->GetSize(&mrc);						mo = GetRectBitmap(&mrc, o);
 			memcpy(&mrkLine, &axline, sizeof(LineDEF));
 			mrkLine.width = axline.width < minw ? minw * 3.0 : axline.width *3.0;
@@ -1112,6 +1125,7 @@ Axis::DoMark(anyOutput *o, bool mark)
 		return;
 		}
 	if(mark){
+		if(mo) DelBitmapClass(mo);					mo = 0L;
 		memcpy(&mrc, &rDims, sizeof(RECT));
 		IncrementMinMaxRect(&mrc, 6 + o->un2ix(axline.width));
 		mo = GetRectBitmap(&mrc, o);
@@ -1263,10 +1277,8 @@ Graph::ExecTool(MouseEvent *mev)
 				if(abs(pc.x -pl.x) > 20 && abs(pc.y -pl.y) >20) {
 					x = ((Graph*)PasteObj)->GRect.Xmax - ((Graph*)PasteObj)->GRect.Xmin;
 					y = ((Graph*)PasteObj)->GRect.Ymax - ((Graph*)PasteObj)->GRect.Ymin;
-					scale.sx.fy = (lfp[1].fx - lfp[0].fx)/x;
-					scale.sy.fy = (lfp[1].fy - lfp[0].fy)/y;
-					//preserve aspect ratio
-					scale.sx.fy = (scale.sx.fy + scale.sy.fy) /2.0;
+					scale.sx.fy = (lfp[1].fx - lfp[0].fx)/x;	scale.sy.fy = (lfp[1].fy - lfp[0].fy)/y;
+					scale.sx.fy = (scale.sx.fy + scale.sy.fy) /2.0;		//preserve aspect ratio
 					scale.sy.fy = scale.sx.fy;
 					}
 				((Graph*)PasteObj)->GRect.Xmax -= ((Graph*)PasteObj)->GRect.Xmin;
@@ -1275,7 +1287,34 @@ Graph::ExecTool(MouseEvent *mev)
 				PasteObj->Command(CMD_SCALE, &scale, 0L);
 				Command(CMD_DROP_GRAPH, (void*)PasteObj, 0L);
 				}
-			else DeleteGO(PasteObj);
+			else {
+				anyOutput *ScaleOut;
+				
+				if(ScaleOut = NewBitmapClass(10, 10, CurrDisp->hres, CurrDisp->vres)) {
+					PasteObj->parent = this;	PasteObj->DoPlot(ScaleOut);
+					switch(PasteObj->Id){
+					case GO_POLYLINE:		case GO_POLYGON:
+						IncrementMinMaxRect(&PasteObj->rDims, -(3*CurrDisp->un2ix(((polyline*)PasteObj)->pgLine.width)+3));
+						break;
+					case GO_BEZIER:
+						IncrementMinMaxRect(&PasteObj->rDims, 3);
+						break;
+						}
+					scale.sx.fx = CurrDisp->fix2un(-PasteObj->rDims.left);
+					scale.sy.fx = CurrDisp->fiy2un(-PasteObj->rDims.top);
+					PasteObj->Command(CMD_SCALE, &scale, 0L);
+					scale.sx.fx = lfp[0].fx;	scale.sy.fx = lfp[0].fy;	scale.sz.fx = 0.0;
+					if(abs(pc.x -pl.x) > 8 && abs(pc.y -pl.y) > 8) {
+						x = CurrDisp->fix2un(PasteObj->rDims.right - PasteObj->rDims.left);
+						y = CurrDisp->fiy2un(PasteObj->rDims.bottom - PasteObj->rDims.top);
+						scale.sx.fy = (lfp[1].fx - lfp[0].fx)/x;	
+						scale.sy.fy = (lfp[1].fy - lfp[0].fy)/y;
+						}
+					PasteObj->Command(CMD_SCALE, &scale, 0L);
+					delete ScaleOut;
+					}
+				Command(CMD_DROP_GRAPH, (void*)PasteObj, 0L);
+				}
 			PasteObj = 0L;
 			ToolMode = TM_STANDARD;			CurrDisp->MouseCursor(MC_ARROW, false);
 			break;
@@ -1344,15 +1383,18 @@ Graph::ExecTool(MouseEvent *mev)
 				memcpy(&pl, &pc, sizeof(POINT));
 				line[1].x = pc.x = mev->x;		line[1].y = pc.y = mev->y;
 				line[0].x = pl.x;				line[0].y = pl.y;
-				CurrDisp->ShowLine(line, 2, color);
 				AddToPolygon(&tl_nPts, tl_pts, &pc);
+#ifdef _WINDOWS
+				CurrDisp->ShowLine(line, 2, color);
+#else
+				CurrDisp->ShowLine(tl_pts, tl_nPts, color);
+#endif
 				return true;
 				}
 			break;
 			}
 		break;
-	case TM_POLYLINE:
-	case TM_POLYGON:
+	case TM_POLYLINE:	case TM_POLYGON:
 		switch (mev->Action) {
 		case MOUSE_LBDOWN:
 			if(!tl_pts) {
@@ -1404,7 +1446,6 @@ Graph::ExecTool(MouseEvent *mev)
 					else i = NumPlots;
 					Undo.SetGO(this, &Plots[i], ToolMode == TM_POLYLINE ?
 						new polyline(this, data, lfp, (int)tl_nPts) :
-//						(GraphObj*) new Bezier(this, data, lfp, (int)tl_nPts) :
 						(GraphObj*) new polygon(this, data, lfp, (int)tl_nPts), 0L);
 					if(Plots[i]){
 						NumPlots = i+1;
@@ -1476,9 +1517,9 @@ Graph::ExecTool(MouseEvent *mev)
 				else new_go = 0L;
 				if(new_go) Undo.SetGO(this, &Plots[i], new_go, 0L);
 				if(Plots[i]){
-					NumPlots = i+1;
-					Plots[i]->DoPlot(CurrDisp);							//init
-					CurrDisp->ShowMark(CurrGO = Plots[i], MRK_GODRAW);				//edit
+					NumPlots = i+1;				CurrDisp->HideMark();
+					Plots[i]->DoPlot(CurrDisp);								//init
+					CurrDisp->ShowMark(CurrGO = Plots[i], MRK_GODRAW);		//edit
 					Plots[i]->moveable = 1;
 					bModified = true;
 					}
